﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using Verse;
using HarmonyLib;
using System.Reflection.Emit;
using UnityEngine;

namespace ChooseYourMedicine
{
    class DrawHediffRow_Transpiler
    {
        //Credits to legodude?: A transpiler in Vanilla Factions Expanded - Ancients was a really helpful example

        //first if statement:
        //we want to render a button which then opens a drowdown menu when clicked for every tendable hediff(group) in a pawns health tab
        /* original code and where we want our stuff:
            foreach (IGrouping<int, Hediff> source in from x in diffs
			group x by x.UIGroupKey)
			{
				int num4 = source.Count<Hediff>();
				string text = source.First<Hediff>().LabelCap;
				if (num4 != 1)
				{
					text = text + " x" + num4.ToString();
				}

                    ********************************************************
                    ******** <-- here we want to insert our stuff ********
                    ********************************************************

				num3 += Text.CalcHeight(text, width);
			}
			b = num3;
			HealthCardUtility.DoRightRowHighlight(new Rect(0f, curY, rect.width, Mathf.Max(a, b)));

         */


        //second if statement:
        //It somehow happens that "Text.CalcHeight(text, width);" gives back different values (text needs 1 line, text needs 2 lines) for the exact same text and width.
        //This leaded to the button to assign med manually was rendered correctly, but you had to click below it to open the dropdown menu.
        //It was reproducible with "cryptosleep sickness (xhours)" and dev mode off. (Using dev quick test in main menu).
        //It took me quite a few hours to trace down what is happening, but rounding the width seems to fix the issue... I dont know why exactly this is happening... 

        public static IEnumerable<CodeInstruction> DrawHediffRowTranspiler(IEnumerable<CodeInstruction> codeInstructions)
        {
            var codes = codeInstructions.ToList();
            var concat = AccessTools.Method(typeof(String), "Concat", new Type[] { typeof(string), typeof(string), typeof(string) });
            bool foundLineNewButton = false;
            bool foundLineRoundWidth = false;
            for (var i = 0; i < codes.Count; i++)
            {
                var code = codes[i];
                yield return code;

                //if (i == 76) 
                if (foundLineNewButton == false && i > 50 && i < 100 && codes[i-5].opcode == OpCodes.Ldstr && codes[i-2].Calls(concat))
                {
                    yield return new CodeInstruction(OpCodes.Ldarg_0);
                    yield return new CodeInstruction(OpCodes.Ldarg_1);
                    yield return new CodeInstruction(OpCodes.Ldarg_3);
                    yield return new CodeInstruction(OpCodes.Ldloc_S, 7);
                    yield return new CodeInstruction(OpCodes.Callvirt, AccessTools.Method(typeof(IEnumerator<Hediff>), "get_Current"));
                    yield return new CodeInstruction(OpCodes.Call, AccessTools.Method(typeof(DrawHediffRow_Transpiler), nameof(Transpiler)));
                    yield return new CodeInstruction(OpCodes.Ldloc_S, 6); //We put this at the end because we use the one from the CalcHeight method. If we start the transpiler 1 line earlier, 
                                                                          //we are still in the if-statement... 
                    foundLineNewButton = true;
                }

                //round local var 'width'
                if (foundLineRoundWidth == false && i > 10 && codes[i + 1].opcode == OpCodes.Stloc_2)
                {
                    yield return new CodeInstruction(OpCodes.Call, AccessTools.Method(typeof(DrawHediffRow_Transpiler), nameof(RoundWidth)));
                    foundLineRoundWidth = true;
                }
            }
        }

        public static float RoundWidth(float width)
        {
            return Mathf.RoundToInt(width);
        }

        public static void Transpiler(float num6, Rect rect, Pawn pawn, ref float curY, IGrouping<BodyPartRecord, Hediff> test)
        {
            if (pawn == null || pawn.Dead || pawn.RaceProps == null || pawn.RaceProps.IsMechanoid)
            {
                return;
            }

            Hediff hediff1 = test.ElementAt(0);
            //if hediff is not tendable, return
            if (!hediff1.def.tendable || hediff1.IsPermanent() || hediff1.FullyImmune() || (hediff1.def == HediffDefOf.MissingBodyPart && hediff1.BleedRate == 0f))
            {
                return;
            }

            float width = rect.width * 0.625f - 45f;
            List<HediffEntry> hediffEntries = HediffEntryList.hediffEntries;

            //create the rect for the button to assign the medicine
            //When rendering the medicine icon (at the end) we make the rect a bit bigger, so the icon is not too small
            Rect buttonAssignMedicineManually = new Rect(rect.width - 66f, curY + num6 + 3f, 16f, 16f);

            //if we hover over the rect (button invisible) it should get highlighted
            Widgets.DrawHighlightIfMouseover(buttonAssignMedicineManually);

            GUI.DrawTexture(buttonAssignMedicineManually, StartUp.AltTexture_Custom);


            //if we right click the rect it should remove the manually assigned medicine for that hediff
            if (Event.current.type == EventType.MouseDown && Event.current.button == 1 && Mouse.IsOver(buttonAssignMedicineManually))
            {
                foreach (Hediff hediff2 in test)
                {
                    HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediff2.GetUniqueLoadID());
                }
            }


            //if we click the rect, a dropdown menu should appear
            else if (Widgets.ButtonInvisible(buttonAssignMedicineManually, true))
            {
                //if modMedicinePatch is loaded
                if (StartUp.modMedicinePatchLoaded)
                {
                    Compatibility_ModMedicinePatch.DrawHediffRow1(test, pawn);
                }
                //if modMedicinePatch is not loaded
                else
                {                    
                    List<FloatMenuOption> list = new List<FloatMenuOption>();
                    for (int i = 0; i < 5; i++)
                    {
                        List<Hediff> test2 = new List<Hediff>();
                        foreach (Hediff asd in test)
                        {
                            test2.Add(asd);
                        }

                        int idx = i;

                        list.Add(new FloatMenuOption(StartUp.stringMedicalCare[i], delegate ()
                        {
                        //idx 0: "Clear": We remove the entry for manual assigned medicine
                        if (idx == 0)
                            {
                                foreach (Hediff hediff2 in test2)
                                {
                                    HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediff2.GetUniqueLoadID());
                                }
                            }

                        //for all others: We add a new entry
                        else
                            {
                                List<FloatMenuOption> list2 = new List<FloatMenuOption>();
                                for (int j = 0; j < 2; j++)
                                {
                                    int idx2 = j;
                                    string testString = "";
                                    if (idx2 == 0) { testString = "Kopp_CYM_TendOneTime".Translate(); }
                                    if (idx2 == 1) { testString = "Kopp_CYM_TendUntilHediffGone".Translate(); }

                                    list2.Add(new FloatMenuOption(testString, delegate ()
                                    {

                                        foreach (Hediff hediff2 in test2)
                                        {
                                        //check, if we already assigned a medicine to that hediff(s)
                                        HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediff2.GetUniqueLoadID());

                                        //add a new entry
                                        HediffEntry entry = new HediffEntry();
                                            entry.hediffUniqueLoadID = hediff2.GetUniqueLoadID();
                                            entry.pawnUniqueLoadID = pawn.GetUniqueLoadID();
                                            entry.medicalCare = (MedicalCareCategory)idx;
                                            if (idx2 == 0) { entry.oneTime = true; }
                                            if (idx2 == 1) { entry.oneTime = false; }
                                            HediffEntryList.hediffEntries.Add(entry);
                                        }

                                    }, j == 0 ? StartUp.randomTextures[1] : StartUp.randomTextures[2], Color.white, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));
                                }
                                Find.WindowStack.Add(new FloatMenu(list2));
                            }
                        }, idx == 0 ? StartUp.randomTextures[0] : Utility_GetList.GetTexture((MedicalCareCategory)idx), Color.white, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));

                    }
                    Find.WindowStack.Add(new FloatMenu(list));
                }
            }


            //The following is to render a icon (medicine) if a medicine got assigned to a hediff manually
            //Instead of checking the list every time (60 per second?) can we somehow cache this?

            //Above we made the button smaller (compared to prior version) but we want the medicine icon not get rendered too small
            buttonAssignMedicineManually = buttonAssignMedicineManually.ExpandedBy(1f);

            string uniqueLoadID = hediff1.GetUniqueLoadID();
            foreach (HediffEntry hediffEntry in hediffEntries)
            {
                //if yes, render the correct icon (type of medicine)
                if (hediffEntry.hediffUniqueLoadID == uniqueLoadID)
                {
                    string tooltip = "";
                    if (hediffEntry.oneTime) { tooltip = "Kopp_CYM_TendOneTime".Translate(); }
                    else { tooltip = "Kopp_CYM_TendUntilHediffGone".Translate(); }
                    TooltipHandler.TipRegion(buttonAssignMedicineManually, tooltip);

                    //if modMedicinePatch is loaded
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        Compatibility_ModMedicinePatch.DrawHediffRow2(buttonAssignMedicineManually, hediffEntry);
                    }
                    //if modMedicinePatch is not loaded
                    else
                    {
                        GUI.DrawTexture(buttonAssignMedicineManually, Utility_GetList.GetTexture(hediffEntry.medicalCare));
                    }
                    

                }
            }
        }
    }   
}
