﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using Verse;
using System.IO;

namespace ChooseYourMedicine
{
    class ModSettings : IExposable
    {
        //the config file
        public static ModSettings settings;

        //Execute when the config is not loaded yet
        public void LoadConfig()
        {
            settings = GetSettings<ModSettings>();

        }

        //a slightly modified copy of vanilla
        public T GetSettings<T>() where T : ModSettings, new()
        {
            if (this.modSettings != null && this.modSettings.GetType() != typeof(T))
            {
                return default(T);
            }
            if (this.modSettings != null)
            {
                return (T)((object)this.modSettings);
            }
            this.modSettings = ReadModSettings<T>();
            this.modSettings.Mod = this;
            return this.modSettings as T;
        }

        //a slightly modified copy of vanilla
        public static T ReadModSettings<T>() where T : ModSettings, new()
        {
            string settingsFilename = GetSettingsFilename();
            T t = default(T);
            try
            {
                if (File.Exists(settingsFilename))
                {
                    Scribe.loader.InitLoading(settingsFilename);
                    try
                    {
                        //Scribe_Deep.Look<T>(ref t, "ModSettings", Array.Empty<object>());
                        Scribe_Deep.Look<T>(ref t, "ModSettings", Array.Empty<object>());
                    }
                    finally
                    {
                        Scribe.loader.FinalizeLoading();
                    }
                }
            }
            catch (Exception ex)
            {
                Log.Warning(string.Format("Caught exception while loading mod settings data for {0}. Generating fresh settings. The exception was: {1}", "Caravan Item Selection Enhanced", ex.ToString()));
                t = default(T);
            }
            if (t == null)
            {
                t = Activator.CreateInstance<T>();
            }
            return t;
        }

        //a slightly modified copy of vanilla
        private static string GetSettingsFilename()
        {
            return Path.Combine(GenFilePaths.ConfigFolderPath, GenText.SanitizeFilename("Mod_Choose Your Medicine_Settings_ChooseYourMedicine.xml"));
        }

        public List<MedicineEntryList> lists;
        //public bool useVanillaPawnsTab = false;

        public void ExposeData()
        {
            Scribe_Collections.Look(ref lists, "lists", LookMode.Deep, Array.Empty<object>());
            //Scribe_Values.Look(ref useVanillaPawnsTab, "useVanillaPawnsTab", false, true);
        }

        //static or not?
        public static void Write()
        {
            WriteModSettings("Choose Your Medicine", "Settings_ChooseYourMedicine", settings);
        }

        //a slightly modified copy of vanilla
        public static void WriteModSettings(string modIdentifier, string modHandleName, ModSettings settings)
        {
            Scribe.saver.InitSaving(GetSettingsFilename(), "SettingsBlock");
            try
            {
                //Scribe_Deep.Look<ModSettings>(ref settings, "ModSettings", Array.Empty<object>());
                Scribe_Deep.Look<ModSettings>(ref settings, "ModSettings", Array.Empty<object>());
            }
            finally
            {
                Scribe.saver.FinalizeSaving();
            }
        }

        public ModSettings Mod { get; internal set; }
        private ModSettings modSettings;
    }
}