﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;
using RimWorld;
using Verse;
using Verse.AI;
using UnityEngine;

namespace ChooseYourMedicine
{
    //this is to be able to assign which medicine should be used for operations... And if the worst to reach 98% surgery success chance (== Vanilla cap) should be used.

    //doctorsMedicalSurgerySuccessChance:   surgeon.GetStatValue(StatDefOf.MedicalSurgerySuccessChance, true)
    //bedsSurgerySuccessChance:             patient.CurrentBed().GetStatValue(StatDefOf.SurgerySuccessChanceFactor, true)
    //medicinesSurgeryChanceFactor:         this is what we want... Recipe_Surgery.MedicineMedicalPotencyToSurgeryChanceFactor.Evaluate(this.GetAverageMedicalPotency(ingredients, bill))
    //                                      needs to be converted to a medicine
    //recipeSurgerySuccessChanceFactor:     this.recipe.surgerySuccessChanceFactor
    //inspiration:                          surgeon.InspirationDef == InspirationDefOf.Inspired_Surgery && !patient.RaceProps.IsMechanoid

    //=> surgerySuccessChance = 1f * doctorsMedicalSurgerySuccessChance * bedsSurgerySuccessChance * medicinesSurgeryChanceFactor * recipeSurgerySuccessChanceFactor (*2f (if inspiration))


    public static class AddEveryMedicineToRelevantThings_Prefix
    {
        public static bool Prefix(Pawn pawn, Thing billGiver, ref List<Thing> relevantThings, Predicate<Thing> baseValidator, Map map)
        {
            Pawn patient = billGiver as Pawn;
            Pawn surgeon = pawn;
            //get the lists of the settings done in the gui
            List<MedicineEntryList> lists = Utility_GetList.GetList();
            //which list should be used?
            int listIndex = Utility_GetList.GetTheCorrectSubList(patient);

            //Get the best medicalCare that is allowed for the patient: Check playerSettings and guiSelection. It is a replacement for WorkGiver_DoBill.GetMedicalCareCategory()
            MedicalCareCategory medicalCare = MedicalCareCategory.Best;
            if (patient != null && patient.playerSettings != null)
            {
                medicalCare = patient.playerSettings.medCare;
                if (medicalCare > lists[listIndex].operationsMaxAllowedMedCare)
                {
                    medicalCare = lists[listIndex].operationsMaxAllowedMedCare;
                }
            }

            int idx = 0;
            if (StartUp.modMedicinePatchLoaded)
            {
                int medicalCareOperation = 0;
                Compatibility_ModMedicinePatch.GetIndexFromMedicalCare(lists[listIndex].operationsMaxAllowedMedCare, ref medicalCareOperation);
                int medicalCarePlayerSettings = 0;
                Compatibility_ModMedicinePatch.GetIndexFromMedicalCare(patient.playerSettings.medCare, ref medicalCarePlayerSettings);
                idx = Math.Min(medicalCareOperation, medicalCarePlayerSettings);

            }



            //If we want to use the worst medicine to reach 0.98f successChance or only operate when we can reach max successChance:
            if (lists[listIndex].operationSmartMedicineChoice || lists[listIndex].operationsOnlyWhenMaxSuccessChance)
            {
                Bill bill = patient.BillStack.FirstShouldDoNow;

                float surgeonSuccessChance = 1f;
                if (!patient.RaceProps.IsMechanoid)
                {
                    surgeonSuccessChance = surgeon.GetStatValue(StatDefOf.MedicalSurgerySuccessChance, true);
                }
                float bedsSuccessChance = 1f;
                if (!bill.recipe.surgeryIgnoreEnvironment && patient.InBed())
                {
                    bedsSuccessChance = patient.CurrentBed().GetStatValue(StatDefOf.SurgerySuccessChanceFactor, true);
                }
                float recipeSuccessChance = bill.recipe.surgerySuccessChanceFactor;
                float inspirationSuccessChance = 1f;
                if (surgeon.InspirationDef == InspirationDefOf.Inspired_Surgery && !patient.RaceProps.IsMechanoid)
                {
                    inspirationSuccessChance = 2f;
                }

                //the necessary success value of the medicine to reach 0.98f
                float necessaryValueForBestChance = 0.98f / (surgeonSuccessChance * bedsSuccessChance * recipeSuccessChance * inspirationSuccessChance);

                //getting the curve to "translate" medicalPotency to surgerySuccessChance
                Type typeInQuestion = typeof(Recipe_Surgery);
                FieldInfo field = typeInQuestion.GetField("MedicineMedicalPotencyToSurgeryChanceFactor", BindingFlags.NonPublic | BindingFlags.Static);
                SimpleCurve curve = new SimpleCurve();
                curve = (SimpleCurve)field.GetValue(null);

                //getting all the available and allowed (medCare) medicine
                List<Thing> tmpMedicine = new List<Thing>();
                List<Thing> list = map.listerThings.ThingsInGroup(ThingRequestGroup.Medicine);
                for (int i = 0; i < list.Count; i++)
                {
                    Thing thing = list[i];
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        if (Compatibility_ModMedicinePatch.CheckIfIdxAllowsMedicine(thing, idx) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            tmpMedicine.Add(thing);
                        }
                    }
                    else
                    {
                        if (medicalCare.AllowsMedicine(thing.def) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            tmpMedicine.Add(thing);
                        }
                    }
                }

                //split all available medicines in "willGrantBestChance" and "wontGrantBestChance"
                List<Thing> medicineWillGrant = new List<Thing>();
                List<Thing> medicineWontGrant = new List<Thing>();
                foreach (Thing medicine in tmpMedicine)
                {
                    float medicinesSurgerySuccessChance = curve.Evaluate(medicine.def.GetStatValueAbstract(StatDefOf.MedicalPotency, null));
                    if (medicinesSurgerySuccessChance >= necessaryValueForBestChance)
                    {                      
                        medicineWillGrant.Add(medicine);
                    }
                    else
                    {
                        medicineWontGrant.Add(medicine);
                    }
                }

                //sort the lists: The list with medicines that grant the 0.98f
                //if "use worst medicine that grants max success: the worst should come first
                if (lists[listIndex].operationSmartMedicineChoice)
                {
                    medicineWillGrant.SortBy((Thing x) => x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));
                }
                //else: the best should come first
                else
                {
                    medicineWillGrant.SortBy((Thing x) => -x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));
                }

                //sort the lists: The list with medicines that do not grant the 0.98f -> the best should come first
                medicineWontGrant.SortBy((Thing x) => -x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));

                //if we only want operations to happen if we reach the max surgery success chance, we only give back the medicines that will grant it
                if (lists[listIndex].operationsOnlyWhenMaxSuccessChance)
                {
                    relevantThings.AddRange(medicineWillGrant);
                }
                //Otherwise: merge the two lists, first the willGrant, then the wontGrant
                else
                {
                    relevantThings.AddRange(medicineWillGrant);
                    relevantThings.AddRange(medicineWontGrant);
                }
            }

            //If we dont want to use the worst to reach at least 0.98f: mostly a copy of vanilla
            else
            {
                List<Thing> list = map.listerThings.ThingsInGroup(ThingRequestGroup.Medicine);
                for (int i = 0; i < list.Count; i++)
                {
                    Thing thing = list[i];
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        if (Compatibility_ModMedicinePatch.CheckIfIdxAllowsMedicine(thing, idx) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            relevantThings.Add(thing);
                        }
                    }
                    else
                    {
                        if (medicalCare.AllowsMedicine(thing.def) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            relevantThings.Add(thing);
                        }
                    }
                }
                relevantThings.SortBy((Thing x) => -x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));
            }
            
            //skip original
            return false;
        }
    }
}

