﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using Verse;
using UnityEngine;

namespace ChooseYourMedicine
{
    public static class Utility_GetList
    {
        public static List<MedicineEntryList> GetList()
        {
            if (ModSettings.settings == null)
            {
                ModSettings settings = new ModSettings();
                settings.LoadConfig();
                if (ModSettings.settings.lists.NullOrEmpty())
                {
                    MedicineEntryListsCreation defaultListsCreation = new MedicineEntryListsCreation();
                    defaultListsCreation.CreateDefaultConfigs();
                    ModSettings.Write();
                }
                lists = ModSettings.settings.lists;
            }
            else
            {
                lists = ModSettings.settings.lists;
            }
            return lists;
        }



        public static int GetTheCorrectSubList(Pawn patient)
        {
            //copied and modified from Pawn_PlayerSettings.ResetMedicalCare()
            int listsIndex = 0;
            if (patient.Faction == Faction.OfPlayer)
            {
                if (patient.RaceProps.Animal)
                {
                    listsIndex = 3;
                    return listsIndex;
                }
                if (patient.IsPrisoner)
                {
                    listsIndex = 1;
                    return listsIndex;
                }
                if (patient.IsSlave)
                {
                    listsIndex = 2;
                    return listsIndex;
                }
                return listsIndex;
            }
            else
            {
                if (patient.Faction == null && patient.RaceProps.Animal)
                {
                    listsIndex = 4;
                    return listsIndex;
                }
                if (patient.Faction == null || !patient.Faction.HostileTo(Faction.OfPlayer))
                {
                    listsIndex = 5;
                    return listsIndex;
                }
                listsIndex = 6;
                return listsIndex;
            }
        }




        public static MedicalCareCategory GetTheCorrectMedicalCareCategory(Pawn patient)
        {
            List<Hediff> tendableHediffs = new List<Hediff>();
            List<Hediff>  allHediffs = patient.health.hediffSet.hediffs;

            for (int i = 0; i < allHediffs.Count; i++)
            {
                if (allHediffs[i].TendableNow(false))
                {
                    tendableHediffs.Add(allHediffs[i]);
                }
            }
            TendUtility.SortByTendPriority(tendableHediffs);

            return GetTheCorrectMedicalCareCategory(patient, tendableHediffs[0]);
        }

        public static MedicalCareCategory GetTheCorrectMedicalCareCategory(Pawn patient, Hediff firstTendedHediff)
        {

            if (HediffEntryList.hediffEntries != null)
            {
                foreach (HediffEntry hediffEntry in HediffEntryList.hediffEntries)
                {
                    if (hediffEntry.pawnUniqueLoadID == patient.GetUniqueLoadID())
                    {
                        if (firstTendedHediff.GetUniqueLoadID() == hediffEntry.hediffUniqueLoadID)
                        {
                            return hediffEntry.medicalCare;
                        }
                    }
                }
            }

            MedicalCareCategory medicalCare = MedicalCareCategory.NoCare;
            if (patient?.playerSettings?.medCare != null)
            {
                medicalCare = patient.playerSettings.medCare;
            }
            List<MedicineEntryList> lists = GetList();
            int correctListIndex = GetTheCorrectSubList(patient);

            int entryIndex = 0;
            foreach (MedicineEntry entry in lists[correctListIndex].list)
            {
                if (firstTendedHediff.def.defName == entry.defName)
                {
                    entryIndex = lists[correctListIndex].list.IndexOf(entry);
                }
            }

            if (firstTendedHediff != null && firstTendedHediff.def.HasComp(typeof(HediffComp_Immunizable)))
            {
                if (firstTendedHediff.TryGetComp<HediffComp_Immunizable>().Immunity * 100f < lists[correctListIndex].list[entryIndex].offSetImmunityLower)
                {
                    medicalCare = lists[correctListIndex].list[entryIndex].careCategoryImmunityLower;
                }
                else if ((firstTendedHediff.TryGetComp<HediffComp_Immunizable>().Immunity - firstTendedHediff.Severity) * 100f < lists[correctListIndex].list[entryIndex].offSetProblematic)
                {
                    medicalCare = lists[correctListIndex].list[entryIndex].careCategoryProblematic;
                }
                else if (firstTendedHediff.TryGetComp<HediffComp_Immunizable>().Immunity * 100f > firstTendedHediff.Severity * 100f + lists[correctListIndex].list[entryIndex].offSetEasy)
                {
                    medicalCare = lists[correctListIndex].list[entryIndex].careCategoryEasy;
                }
                else
                {
                    medicalCare = lists[correctListIndex].list[entryIndex].careCategoryStandard;
                }
            }
            else
            {
                medicalCare = lists[correctListIndex].list[entryIndex].careCategoryStandard;
            }
            //if modMedicinePatch is loaded
            if (StartUp.modMedicinePatchLoaded)
            {
                return Compatibility_ModMedicinePatch.GetList1(medicalCare, patient);
            }
            //if modMedicinePatch is not loaded
            else
            {
                if (patient != null && patient.playerSettings != null && medicalCare > patient.playerSettings.medCare)
                {
                    medicalCare = patient.playerSettings.medCare;
                }

                return medicalCare;
            }
        }


        public static Texture2D GetTexture(MedicalCareCategory medicalCare)
        {
            if (medicalCare == MedicalCareCategory.Best)
            {
                return StartUp.careTextures[4];
            }
            else if (medicalCare == MedicalCareCategory.NormalOrWorse)
            {
                return StartUp.careTextures[3];
            }
            else if (medicalCare == MedicalCareCategory.HerbalOrWorse)
            {
                return StartUp.careTextures[2];
            }
            else
            {
                return StartUp.careTextures[1];
            }
        }


        public static List<MedicineEntryList> lists = new List<MedicineEntryList>();
    }
}
