﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using RimWorld;
using UnityEngine;

namespace ChooseYourMedicine
{
    public static class DrawButton
    {

        public static void DrawButtonToAssignMedManually(Rect buttonRect, Pawn pawn, List<Hediff> hediffList, Hediff hediff)
        {
            List<HediffEntry> hediffEntries = HediffEntryList.hediffEntries;

            //Draw/highlight the button
            Widgets.DrawHighlightIfMouseover(buttonRect);
            GUI.DrawTexture(buttonRect, StartUp.AltTexture_Custom);

            //if we right click the rect, it should remove the manually assigned medicine for that hediff
            if (Event.current.type == EventType.MouseDown && Event.current.button == 1 && Mouse.IsOver(buttonRect))
            {
                foreach (Hediff hediff2 in hediffList)
                {
                    HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediff2.GetUniqueLoadID());
                }
            }

            //if mouse is up (not pressed), we are not drag painting
            if (Event.current.rawType == EventType.MouseUp/* || Input.GetMouseButtonUp(0)*/)
            {
                dropdownPainting = false;
            }

            //check if we are drag painting
            Widgets.DraggableResult draggableResult;
            draggableResult = Widgets.ButtonInvisibleDraggable(buttonRect, false);

            //if we are left clicking: dropdown menu should appear
            if (draggableResult == Widgets.DraggableResult.Pressed)
            {
                if (StartUp.modMedicinePatchLoaded)
                {
                    Compatibility_ModMedicinePatch.DrawHediffRow1(hediffList, pawn);
                }
                else
                {
                    MakeDropdownMenu(pawn, hediffList);
                }
            }

            else
            {
                //if we are left clicking and holding (== dragPainting): put the necessary infos we need for 'pasting' into vars and return
                //return is needed, as otherwise we 'copy' the infos from where the cursor is right now while dragging, but we only want to copy when starting to drag
                if (draggableResult == Widgets.DraggableResult.Dragged)
                {
                    dropdownPainting = true;
                    foreach (HediffEntry hediffEntry in hediffEntries)
                    {
                        if (hediffEntry.hediffUniqueLoadID == hediff.GetUniqueLoadID())
                        {
                            dropdownMedCare = hediffEntry.medicalCare;
                            dropdownOnce = hediffEntry.oneTime;
                        }
                    }
                    return;
                }

                //if dropdownPainting is still true, render the 'copied' infos on screen and 'paste' the copied infos 'to' another hediff if necessary (when hovering over another button)
                if (dropdownPainting)
                {
                    Texture texture = StartUp.modMedicinePatchLoaded ? Compatibility_ModMedicinePatch.GetCareTexture(dropdownMedCare) : Utility_GetList.GetTexture(dropdownMedCare);
                    DragPainting.RenderClipboardInfos(texture, dropdownOnce);

                    //Use the dropdownSelection to set treatment for other hediff when hovering over button
                    //how can we make that pasting only happens once and not for each frame?
                    if (Mouse.IsOver(buttonRect))
                    {
                        foreach (Hediff hediff2 in hediffList)
                        {
                            MakeNewHediffEntry(hediff2, pawn, dropdownMedCare, dropdownOnce);
                        }
                    }
                }
            }

            buttonRect = buttonRect.ExpandedBy(1f);
            RenderCorrectMedTexture(hediffEntries, hediff.GetUniqueLoadID(), buttonRect);
        }


        public static void MakeNewHediffEntry(Hediff hediff, Pawn pawn, MedicalCareCategory medCare, bool once)
        {
            string hediffID = hediff.GetUniqueLoadID();
            //check if we already have the same exact entry in the list
            if (HediffEntryList.hediffEntries.Where(x => x.hediffUniqueLoadID == hediffID && x.medicalCare == medCare && x.oneTime == once && x.pawnUniqueLoadID == pawn.GetUniqueLoadID()).Any())
            {
                return;
            }

            //remove entries from list that already have a medCare assigned to
            HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediffID);

            //add a new entry
            HediffEntry entry = new HediffEntry();
            entry.hediffUniqueLoadID = hediffID;
            entry.pawnUniqueLoadID = pawn.GetUniqueLoadID();
            entry.medicalCare = medCare;
            entry.oneTime = once;
            HediffEntryList.hediffEntries.Add(entry);
        }


        public static bool RenderButton(Pawn pawn, Hediff hediff)
        {
            if (pawn == null || pawn.Dead || pawn.RaceProps == null || pawn.RaceProps.IsMechanoid ||
                !hediff.def.tendable || hediff.IsPermanent() || hediff.FullyImmune() || (hediff.def == HediffDefOf.MissingBodyPart && hediff.BleedRate == 0f))
            {
                return false;
            }
            else
            {
                return true;
            }
        }


        public static void RenderCorrectMedTexture(List<HediffEntry> hediffEntries, string uniqueLoadID, Rect buttonRect)
        {
            foreach (HediffEntry hediffEntry in hediffEntries)
            {
                if (hediffEntry.hediffUniqueLoadID == uniqueLoadID)
                {
                    string tooltip = hediffEntry.oneTime ? "Kopp_CYM_TendOneTime".Translate() : "Kopp_CYM_TendUntilHediffGone".Translate();
                    TooltipHandler.TipRegion(buttonRect, tooltip);

                    if (StartUp.modMedicinePatchLoaded)
                    {
                        Compatibility_ModMedicinePatch.DrawHediffRow2(buttonRect, hediffEntry);
                    }
                    else
                    {
                        GUI.DrawTexture(buttonRect, Utility_GetList.GetTexture(hediffEntry.medicalCare));
                    }
                }
            }
        }


        public static void MakeDropdownMenu(Pawn pawn, List<Hediff> hediffList)
        {
            List<FloatMenuOption> list = new List<FloatMenuOption>();
            for (int i = 0; i < 5; i++) //vanilla: 5 options: clear, noMedicine, herbal, industrial, glitter
            {
                int idx = i;

                list.Add(new FloatMenuOption(StartUp.stringMedicalCare[i].CapitalizeFirst(), delegate ()
                {
                    //idx 0: "Clear": We remove the entry for manual assigned medicine
                    if (idx == 0)
                    {
                        foreach (Hediff hediff2 in hediffList)
                        {
                            HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediff2.GetUniqueLoadID());
                        }
                    }

                    //for all others: We add a new entry (and thereby eventually remove a entry if we already had medicine assigned to that hediff)
                    else
                    {
                        List<FloatMenuOption> list2 = new List<FloatMenuOption>();
                        for (int j = 0; j < 2; j++)
                        {
                            int idx2 = j;
                            string stringTendOnce = idx2 == 0 ? "Kopp_CYM_TendOneTime".Translate() : "Kopp_CYM_TendUntilHediffGone".Translate();

                            list2.Add(new FloatMenuOption(stringTendOnce, delegate ()
                            {
                                foreach (Hediff hediff2 in hediffList)
                                {
                                    MakeNewHediffEntry(hediff2, pawn, (MedicalCareCategory)idx, idx2 == 0);
                                }

                            }, j == 0 ? StartUp.randomTextures[1] : StartUp.randomTextures[2], Color.white, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));
                        }
                        Find.WindowStack.Add(new FloatMenu(list2));
                    }
                }, idx == 0 ? StartUp.randomTextures[0] : Utility_GetList.GetTexture((MedicalCareCategory)idx), Color.white, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));

            }
            Find.WindowStack.Add(new FloatMenu(list));
        }


        public static bool dropdownPainting;
        public static MedicalCareCategory dropdownMedCare;
        public static bool dropdownOnce;
    }
}
