﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using Verse.AI;
using RimWorld;
using RimWorld.Planet;
using UnityEngine;

namespace ChooseYourMedicine
{
    public class Compatibility_ModMedicinePatch
    {
        public static void CreateOrderedCareList()
        {
            orderedCareList = TraverseStuff.method_modMedicinePatch_GetOrderedCareList.GetValue<List<MedicalCareCategory>>();
        }


        //I think I made this in case mod added medicines get removed from the save to not throw errors
        public static void TryToFixGuiEntries()
        {
            List<MedicineEntryList> lists = Utility_GetList.GetList();
            int count = orderedCareList.Count();
            foreach (MedicineEntryList list in lists)
            {
                foreach (MedicineEntry entry in list.list)
                {
                    if ((int)entry.careCategoryEasy > count - 1)
                    {
                        entry.careCategoryEasy = MedicalCareCategory.NormalOrWorse;
                    }
                    if ((int)entry.careCategoryStandard > count - 1)
                    {
                        entry.careCategoryStandard = MedicalCareCategory.NormalOrWorse;
                    }
                    if ((int)entry.careCategoryProblematic > count - 1)
                    {
                        entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
                    }
                    if ((int)entry.careCategoryImmunityLower > count - 1)
                    {
                        entry.careCategoryImmunityLower = MedicalCareCategory.NormalOrWorse;
                    }
                }
                if ((int)list.operationsMaxAllowedMedCare > count - 1)
                {
                    list.operationsMaxAllowedMedCare = MedicalCareCategory.NormalOrWorse;
                }
            }
        }


        //necessary? Seems like if we have mod added medicines assigned manually to a hediff and load the save with MMP it gets changed to "no medicine". Which should be fine. Edge case.
        //But it may be problematic if mod added medicines that are assigned to a pawn manually get removed from the save
        //=> Yes, throws red errors, but doc will tend without medicine, so problem will fix itself somehow.
        //this has to be done when loading a save... Because the list is empty on RW startup... Because there is a list for each savegame...
        public static void TryToFixManuallyAssignedEntries()
        {
            int count = orderedCareList.Count();
            foreach (HediffEntry hediffEntry in HediffEntryList.hediffEntries)
            {
                if ((int)hediffEntry.medicalCare > count -1)
                {
                    hediffEntry.medicalCare = MedicalCareCategory.NormalOrWorse;
                }
            }
        }


        public static void GetIndexFromMedicalCare(MedicalCareCategory medicalCare, ref int idx)
        {
            for (int i = 0; i < Compatibility_ModMedicinePatch.orderedCareList.Count(); i++)
            {
                if (medicalCare == Compatibility_ModMedicinePatch.orderedCareList[i])
                {
                    idx = i;
                }
            }
        }


        public static bool CheckIfMedicineFitsCareCategory(Thing medicine, int idx)
        {
            if (medicine != null && medicine.def == TraverseStuff.modMedicineList[idx].thingDef)
            {
                return true;
            }
            else
            {
                return false;
            }
        }


        public static bool CheckIfIdxAllowsMedicine(Thing medicine, int idx)
        {
            if (medicine != null && medicine.def != null && medicine.def.GetStatValueAbstract(StatDefOf.MedicalPotency, null) <=
                (TraverseStuff.modMedicineList[idx].potency))
            {
                return true;
            }
            else
            {
                return false;
            }
        }


        public static bool Validator(Thing m, Pawn healer, Pawn patient, int idx)
        {
            return !m.IsForbidden(healer)
                && m.def.IsMedicine
                //&& (patient.playerSettings == null || m.def == ModMedicinePatch.ModMedicinePatch.GetMedicineByCare(Compatibility_ModMedicinePatch.orderedCareList[idx]).thingDef)
                && (patient.playerSettings == null || CheckIfMedicineFitsCareCategory(m, idx))
                && healer.CanReserve(m, 10, 1, null, false);
        }


        public static void DrawHediffRow1(List<Hediff> hediffList, Pawn pawn)
        {
            List<FloatMenuOption> list = new List<FloatMenuOption>();
            List<MedicalCareCategory> medicalCareCategories = Compatibility_ModMedicinePatch.orderedCareList;
            List<string> stringMedicalCare = new List<string>();
            stringMedicalCare.Add("Kopp_CYM_Clear".Translate());

            for (int i = 1; i < medicalCareCategories.Count(); i++)
            {
                stringMedicalCare.Add(medicalCareCategories[i].GetLabel());
            }
            for (int i = 0; i < medicalCareCategories.Count(); i++)
            {
                int idx = i;

                list.Add(new FloatMenuOption(stringMedicalCare[idx].CapitalizeFirst(), delegate ()
                {
                    //idx 0: "Clear": We remove the entry for manual assigned medicine
                    if (idx == 0)
                    {
                        foreach (Hediff hediff2 in hediffList)
                        {
                            HediffEntryList.hediffEntries.RemoveAll(item => item.hediffUniqueLoadID == hediff2.GetUniqueLoadID());
                        }
                    }

                    //for all others: We add a new entry
                    else
                    {
                        List<FloatMenuOption> list2 = new List<FloatMenuOption>();
                        for (int j = 0; j < 2; j++)
                        {
                            int idx2 = j;
                            string testString = "";
                            if (idx2 == 0) { testString = "Kopp_CYM_TendOneTime".Translate(); }
                            if (idx2 == 1) { testString = "Kopp_CYM_TendUntilHediffGone".Translate(); }

                            list2.Add(new FloatMenuOption(testString, delegate ()
                            {

                                foreach (Hediff hediff2 in hediffList)
                                {
                                    DrawButton.MakeNewHediffEntry(hediff2, pawn, medicalCareCategories[idx], idx2 == 0);
                                }

                            }, j == 0 ? StartUp.randomTextures[1] : StartUp.randomTextures[2], Color.white, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));
                        }
                        Find.WindowStack.Add(new FloatMenu(list2));
                    }
                }, idx == 0 ? StartUp.randomTextures[0] : TraverseStuff.modMedicineList[idx].tex, Color.white, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));

            }
            Find.WindowStack.Add(new FloatMenu(list));

        }

        public static void DrawHediffRow2(Rect buttonAssignMedicineManually, HediffEntry hediffEntry)
        {
            GUI.DrawTexture(buttonAssignMedicineManually, GetCareTexture(hediffEntry.medicalCare));
        }

        public static Texture GetCareTexture(MedicalCareCategory medCare)
        {
            return TraverseStuff.method_modMedicinePatch_GetCareTexture.GetValue<Texture2D>(medCare);
        }



        public static void MainTabWindow1(Rect rectStandardDropdown, ref MedicineEntry entry, Rect rectEasyDropdown, Rect rectEasySlider, Rect rectProblematicDropdown, Rect rectProblematicSlider,
            Rect rectImmunityDropdown, Rect rectImmunitySlider)
        {
            MedicalCareDropdownMenu_ModMedicinePatch.DynamicMedicalCareSelectButtonStandard(rectStandardDropdown, entry);
            if (entry.hasCompImmunizable)
            {
                MedicalCareDropdownMenu_ModMedicinePatch.DynamicMedicalCareSelectButtonEasy(rectEasyDropdown, entry);
                entry.offSetEasy = (int)Widgets.HorizontalSlider(rectEasySlider, entry.offSetEasy, 0f, 100f, false, (entry.offSetEasy.ToString() + " %").Colorize(MainTabWindow_Medicine.yellowCustom), "0", "100");

                MedicalCareDropdownMenu_ModMedicinePatch.DynamicMedicalCareSelectButtonProblematic(rectProblematicDropdown, entry);
                entry.offSetProblematic = (int)Widgets.HorizontalSlider(rectProblematicSlider, entry.offSetProblematic, 0f, 100f, false, (entry.offSetProblematic.ToString() + " %").Colorize(MainTabWindow_Medicine.yellowCustom), "0", "100");

                MedicalCareDropdownMenu_ModMedicinePatch.DynamicMedicalCareSelectButtonImmunityLower(rectImmunityDropdown, entry);
                entry.offSetImmunityLower = (int)Widgets.HorizontalSlider(rectImmunitySlider, entry.offSetImmunityLower, 0f, 100f, false, (entry.offSetImmunityLower.ToString() + " %").Colorize(MainTabWindow_Medicine.yellowCustom), "0", "100");
            }
        }


        public static MedicalCareCategory GetList1(MedicalCareCategory medicalCare, Pawn patient)
        {
            List<ModMedicineCopy> localMedList = TraverseStuff.modMedicineList;
            int medicalCareIndex = 0;
            for (int i = 0; i < localMedList.Count(); i++)
            {
                MedicalCareCategory medCare = localMedList[i].care;
                if (medCare == medicalCare)
                {
                    medicalCareIndex = i;
                    break;
                }
            }
            if (patient != null && patient.playerSettings != null)
            {
                int playerMedicalCareIndex = 0;
                for (int i = 0; i < localMedList.Count(); i++)
                {
                    MedicalCareCategory medCare = localMedList[i].care;
                    if (medCare == patient.playerSettings.medCare)
                    {
                        playerMedicalCareIndex = i;
                        break;
                    }
                }
                if (medicalCareIndex > playerMedicalCareIndex)
                {
                    medicalCareIndex = playerMedicalCareIndex;
                }
            }
            return localMedList[medicalCareIndex].care;
        }


        public static List<MedicalCareCategory> orderedCareList;
    }
}
