﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using UnityEngine;
using RimWorld;

namespace ChooseYourMedicine
//copied and modified from https://rimworldwiki.com/wiki/Modding_Tutorials/ModSettings
{
    public class ModSettings_CYM : Verse.ModSettings
    {

        public bool showMainTabButton = true;
        public int maxMedicineCarryCount = 3;
        public int tendingsSearchRange = -1;

        public override void ExposeData()
        {
            Scribe_Values.Look<bool>(ref showMainTabButton, "showMainTabButton", true, true);
            Scribe_Values.Look<int>(ref maxMedicineCarryCount, "maxMedicineCarryCount", 3, true);
            Scribe_Values.Look<int>(ref tendingsSearchRange, "tendingsSearchRange", -1, true);
            base.ExposeData();
        }
    }
    public class Settings_CYM : Mod
    {
        public ModSettings_CYM settings;
        Vector2 scrollPosition;
        public Settings_CYM(ModContentPack content) : base(content)
        {
            this.settings = GetSettings<ModSettings_CYM>();
        }
        public override void DoSettingsWindowContents(Rect inRect)
        {

            Rect rect = new Rect(0f, 0f, inRect.width - 48f, inRect.height);
            Widgets.BeginScrollView(inRect, ref scrollPosition, rect, true);

            Listing_Standard listingStandard = new Listing_Standard();
            listingStandard.Begin(rect);

            listingStandard.Gap(10f);
            listingStandard.GapLine(4f);

            //option to disable the mainTabButton
            //It would be better if the if/else statement only gets executed if the checkbox got clicked, but since this is a simple gui (and game paused) it should not be important
            listingStandard.Gap(40f);
            listingStandard.CheckboxLabeled("Kopp_CYM_ShowMainButton".Translate(), ref settings.showMainTabButton, "Kopp_CYM_ShowMainButtonTooltip".Translate());
            MainButtonDef modsMainButton = CYMDefOf.CYM_Medicine;
            if (modsMainButton == null)
            {
                Log.Warning("ChooseYourMedicine: Could not find main button def.");
            }
            else
            {
                if (modsMainButton.buttonVisible != settings.showMainTabButton)
                {
                    modsMainButton.buttonVisible = settings.showMainTabButton;
                }
            }


            //option to set the amount of medicine a pawn can carry
            listingStandard.Gap(20f);
            listingStandard.Label("Kopp_CYM_AmountMedicineCarry".Translate() + settings.maxMedicineCarryCount.ToString().Colorize(MainTabWindow_Medicine.yellowCustom), -1, "Kopp_CYM_AmountMedicineCarryTooltip".Translate());
            settings.maxMedicineCarryCount = (int)listingStandard.Slider(settings.maxMedicineCarryCount, 1, 50);
            if (InventoryStockGroupDefOf.Medicine.max != settings.maxMedicineCarryCount)
            {
                InventoryStockGroupDefOf.Medicine.max = settings.maxMedicineCarryCount;
            }


            //search radius for medicine when tending
            listingStandard.Gap(20f);
            listingStandard.Label("Kopp_CYM_TendingsSearchRange".Translate() + settings.tendingsSearchRange.ToString().Colorize(MainTabWindow_Medicine.yellowCustom), -1, "Kopp_CYM_TendingsSearchRangeTooltip".Translate());
            settings.tendingsSearchRange = (int)listingStandard.Slider(settings.tendingsSearchRange, -1, 500);


            //Button to open the mods gui from the mod settings
            //seems to work, but RimWorld.MainTabWindow.PostOpen() spits a error. We could change our MainTabWindow_Medicine to have a "public override void PostOpen()" which does not call base.PostOpen()
            //but its easier to just catch the error
            listingStandard.Gap(40f);
            if (listingStandard.ButtonText("Kopp_CYM_OpenModsGui".Translate(), null, 0.4f))
            {
                MainTabWindow_Medicine mainTabWindowModSettings = new MainTabWindow_Medicine()
                {
                    focusWhenOpened = true,
                    doCloseButton = true,
                    doCloseX = true,
                    layer = WindowLayer.Dialog,
                    soundAppear = SoundDefOf.DialogBoxAppear
                };

                //try/catch block to catch the exception
                try
                {
                    Find.WindowStack.Add(mainTabWindowModSettings);
                }
                //since the exception happens when "Find.WindowStack.Add(...)" gets called and we need to change the position of the window after that call, we put the change of the position in the catch block
                catch
                {
                    mainTabWindowModSettings.windowRect.x = (UI.screenWidth - mainTabWindowModSettings.windowRect.width) * 0.5f;
                    mainTabWindowModSettings.windowRect.y = (UI.screenHeight - mainTabWindowModSettings.windowRect.height) * 0.5f;
                }
            }

            listingStandard.End();
            Widgets.EndScrollView();
            base.DoSettingsWindowContents(rect);
        }

        public override string SettingsCategory()
        {
            return "Kopp_CYM_ChooseYourMedicine".Translate();
        }
    }
}