﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using RimWorld;
using UnityEngine;

namespace ChooseYourMedicine
{
	public class Dialog_Operations : Window
	{
		public override Vector2 InitialSize
		{
			get
			{
				//if ideology is not installed, we need one line less, so the height can be smaller (rowHeight + gapHeight) -> no line for "enslaved colonists"
				return new Vector2(1200f, ModLister.IdeologyInstalled == true ? 550f : 505f);
			}
		}

		public Dialog_Operations(List<MedicineEntryList> lists)
		{
			this.forcePause = true;
			this.doCloseX = true;
			this.doCloseButton = true;
			this.absorbInputAroundWindow = true;
			this.doWindowBackground = true;
			this.drawShadow = false;
			this.focusWhenOpened = true; //what does this?
			this.lists = lists;
		}

		//The rect that gets used here is the InitialSize contractedBy(Margin). Margin should be 18f...
		public override void DoWindowContents(Rect inRect)
		{
			float tmpY = 0f;
			float rowHeight = 30f;
			float rowHeight2 = 80f;
			float labelLength = 150f;
			float labelLength2 = 200f;
			float gapHeight = 15f;
			float xValueFirstColumn = 200f;
			float xValueSecondColumn = 450f;
			float xValueThirdColumn = 700f;
			float xValueFourthColumn = 950f;
			float widthHeightOfMedIcons = 26f;
			float sizeCheckbox = 20f;


			//Line for headline
			Text.Font = GameFont.Medium;
			Vector2 textVector = Text.CalcSize("Kopp_CYM_OperationsHeadline".Translate());
			float textWidth = textVector.x;
			float textHeight = textVector.y;

			Rect headline = new Rect(InitialSize.x * 0.5f - 0.5f * textWidth, 0f, textWidth, textHeight);
			Widgets.Label(headline, "Kopp_CYM_OperationsHeadline".Translate());
			tmpY += textHeight + gapHeight;

            Text.Font = GameFont.Small;
            Text.Anchor = TextAnchor.MiddleCenter;


			//we make a single row for the labels of the available options
			Rect rectMaxCareLabel = new Rect(xValueFirstColumn, tmpY, labelLength2, rowHeight2);
			Widgets.Label(rectMaxCareLabel, "Kopp_CYM_OperationsMaxCare".Translate());
			TooltipHandler.TipRegion(rectMaxCareLabel, "Kopp_CYM_OperationsMaxCareTooltip".Translate());

			Rect rectWorstMedicine = new Rect(xValueSecondColumn, tmpY, labelLength2, rowHeight2);
			Widgets.Label(rectWorstMedicine, "Kopp_CYM_OperationsWorseMed".Translate());
            TooltipHandler.TipRegion(rectWorstMedicine, "Kopp_CYM_OperationsWorseMedTooltip".Translate());

			Rect rectOperateOnlyMaxChance = new Rect(xValueThirdColumn, tmpY, labelLength2, rowHeight2);
			Widgets.Label(rectOperateOnlyMaxChance, "Kopp_CYM_OperationsOnlyWhenBestSuccess".Translate());
			TooltipHandler.TipRegion(rectOperateOnlyMaxChance, "Kopp_CYM_OperationsOnlyWhenBestSuccessTooltip".Translate());

			Rect rectBestMedicineForCertainOperations = new Rect(xValueFourthColumn, tmpY, labelLength2, rowHeight2);
			Widgets.Label(rectBestMedicineForCertainOperations, "Kopp_CYM_OperationsUseBest".Translate());
			TooltipHandler.TipRegion(rectBestMedicineForCertainOperations, "Kopp_CYM_OperationsUseBestTooltip".Translate());

			tmpY += rowHeight + 4f * gapHeight;



			//for every pawn category we make a row
			for (int i = 0; i < lists.Count(); i++)
            {
				//if ideology is not installed, we skip the "enslaved colonists" row
				if (i == 2 && !ModLister.IdeologyInstalled)
                {
					continue;
                }

				float tmpX = 0f;
				MedicineEntryList list = lists[i];

				//label of the pawn category
				Rect rectListName = new Rect(tmpX, tmpY, labelLength, rowHeight);
				Widgets.Label(rectListName, list.nameOfTheList);
				tmpX = xValueFirstColumn + 0.5f * labelLength2 - 0.5f * widthHeightOfMedIcons;
				//we want a tooltip for the "imprisoned colonists" label
				if (i == 1)
                {
					TooltipHandler.TipRegion(rectListName, "Kopp_CYM_ImprisonedColonistsTooltip".Translate());
                }

				//dropdown menu for max allowed med care
				Rect dropdownMaxAllowedMedicine = new Rect(tmpX, tmpY, widthHeightOfMedIcons, widthHeightOfMedIcons);
                if (StartUp.modMedicinePatchLoaded)
                {
                    MedicalCareDropdownMenu_ModMedicinePatch.DynamicMedicalCareSelectButtonOperation(dropdownMaxAllowedMedicine, list);
                }
                else
                {
                    MedicalCareDropdownMenu.MedicalCareSelectButtonOperation(dropdownMaxAllowedMedicine, list);
                }
				tmpX = xValueSecondColumn + 0.5f * labelLength2 - 0.5f * sizeCheckbox;

				//checkbox for trying to take worse medicine
				Vector2 vector = new Vector2(tmpX, tmpY);
                Widgets.Checkbox(vector, ref list.operationSmartMedicineChoice, sizeCheckbox);
				tmpX = xValueThirdColumn + 0.5f * labelLength2 - 0.5f * sizeCheckbox;

				//checkbox for operate only when max chance
				Vector2 vector2 = new Vector2(tmpX, tmpY);
				Widgets.Checkbox(vector2, ref list.operationsOnlyWhenMaxSuccessChance, sizeCheckbox);
				tmpX = xValueFourthColumn + 0.5f * labelLength2 - 0.5f * sizeCheckbox;

				//checkbox if best medicine should be used for certain (hardcoded) operations
				Vector2 vector3 = new Vector2(tmpX, tmpY);
				Widgets.Checkbox(vector3, ref list.operationsUseBest, sizeCheckbox);

				tmpY += rowHeight + gapHeight;
			}

			Text.Anchor = TextAnchor.UpperLeft;
			Text.Font = GameFont.Medium;
		}

		public List<MedicineEntryList> lists;

	}

}