﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Reflection;
using RimWorld;
using Verse;
using Verse.AI;
using UnityEngine;
using HarmonyLib;

namespace ChooseYourMedicine
{
    //this is to be able to assign which medicine should be used for operations... And if the worst to reach 98% surgery success chance (== Vanilla cap) should be used. And other mod settings.

    //doctorsMedicalSurgerySuccessChance:   surgeon.GetStatValue(StatDefOf.MedicalSurgerySuccessChance, true)
    //bedsSurgerySuccessChance:             patient.CurrentBed().GetStatValue(StatDefOf.SurgerySuccessChanceFactor, true)
    //medicinesSurgeryChanceFactor:         this is what we want... Recipe_Surgery.MedicineMedicalPotencyToSurgeryChanceFactor.Evaluate(this.GetAverageMedicalPotency(ingredients, bill))
    //                                      needs to be converted to a medicine
    //recipeSurgerySuccessChanceFactor:     this.recipe.surgerySuccessChanceFactor
    //inspiration:                          surgeon.InspirationDef == InspirationDefOf.Inspired_Surgery && !patient.RaceProps.IsMechanoid

    //=> surgerySuccessChance = 1f * doctorsMedicalSurgerySuccessChance * bedsSurgerySuccessChance * medicinesSurgeryChanceFactor * recipeSurgerySuccessChanceFactor (*2f (if inspiration))


    public static class AddEveryMedicineToRelevantThings_Prefix
    {
        public static bool Prefix(Pawn pawn, Thing billGiver, ref List<Thing> relevantThings, Predicate<Thing> baseValidator, Map map)
        {
            //following if-statement assures compatibility with "What the hack"
            if ((billGiver as Pawn)?.RaceProps?.IsMechanoid ?? false)
            {
                return true;
            }

            Pawn patient = billGiver as Pawn;
            Pawn surgeon = pawn;
            //get the lists of the settings done in the gui
            List<MedicineEntryList> lists = Utility_GetList.GetList();
            //which list should be used?
            int listIndex = Utility_GetList.GetTheCorrectSubList(patient);

            //Get the best medicalCare that is allowed for the patient: Check playerSettings and guiSelection. It is a replacement for WorkGiver_DoBill.GetMedicalCareCategory()
            MedicalCareCategory medicalCare = MedicalCareCategory.Best;
            if (patient != null && patient.playerSettings != null)
            {
                medicalCare = patient.playerSettings.medCare;
                if (medicalCare > lists[listIndex].operationsMaxAllowedMedCare)
                {
                    medicalCare = lists[listIndex].operationsMaxAllowedMedCare;
                }
            }

            int idx = 0;
            if (StartUp.modMedicinePatchLoaded)
            {
                int medicalCareOperation = 0;
                Compatibility_ModMedicinePatch.GetIndexFromMedicalCare(lists[listIndex].operationsMaxAllowedMedCare, ref medicalCareOperation);
                int medicalCarePlayerSettings = 0;
                Compatibility_ModMedicinePatch.GetIndexFromMedicalCare(patient.playerSettings.medCare, ref medicalCarePlayerSettings);
                idx = Math.Min(medicalCareOperation, medicalCarePlayerSettings);

            }

            bool anyOutcomeWithQualityDurationCurve = (patient.BillStack.FirstShouldDoNow as Bill_Medical).recipe.surgeryOutcomeEffect?.outcomes?.Where(x => x.GetType() == typeof(SurgeryOutcome_HediffWithDuration) &&
            (x as SurgeryOutcome_HediffWithDuration).qualityToDurationDaysCurve != null).Any() ?? false;


            //Check the operations settings done in the gui:
            //Execute this if either "use worst med to reach 0.98f" or "operate only when max success" is true,
            //as long as the surgery does not cause a hediff whichs duration depends on the used med and the settings in the operations gui state to check that
            //and we have a comp medicineQuality
            if ((lists[listIndex].operationSmartMedicineChoice || lists[listIndex].operationsOnlyWhenMaxSuccessChance) &&
                !(anyOutcomeWithQualityDurationCurve && lists[listIndex].operationsUseBest) &&
                patient?.BillStack?.FirstShouldDoNow?.recipe?.surgeryOutcomeEffect != null &&
                patient.BillStack.FirstShouldDoNow.recipe.surgeryOutcomeEffect.comps.Where(x => x.GetType() == typeof(SurgeryOutcomeComp_MedicineQuality)).Any())

            {
                Bill bill = patient.BillStack.FirstShouldDoNow;
                Bill_Medical billMedical = bill as Bill_Medical;
                BodyPartRecord part = billMedical.Part;

                //getting the needed success chance for the medicine if we want to reach at least 0.98f success chance.
                float necessaryValueForBestChance = 0.98f / GetSuccessChanceIgnoreMedicine(patient.BillStack.FirstShouldDoNow.recipe, surgeon, patient, part, patient.BillStack.FirstShouldDoNow);

                //getting the curve to "translate" medicalPotency to surgerySuccessChance:
                SurgeryOutcomeComp surgeryOutcomeComp = bill.recipe.surgeryOutcomeEffect.comps.Where(x => x.GetType() == typeof(SurgeryOutcomeComp_MedicineQuality)).First();
                SurgeryOutcomeComp_MedicineQuality surgeryOutcomeComp_MedicineQuality = surgeryOutcomeComp as SurgeryOutcomeComp_MedicineQuality;
                SimpleCurve curve = new SimpleCurve();
                if (surgeryOutcomeComp_MedicineQuality?.curve != null)
                {
                    curve = surgeryOutcomeComp_MedicineQuality.curve;
                }
                //if curve is null, use the default curve
                else
                {
                    curve = new SimpleCurve
                    {
                        {
                            new CurvePoint(0f, 0.7f),
                            true
                        },
                        {
                            new CurvePoint(1f, 1f),
                            true
                        },
                        {
                            new CurvePoint(2f, 1.3f),
                            true
                        }
                    };
                }

                //getting all the available and allowed (medCare) medicine
                List<Thing> tmpMedicine = new List<Thing>();
                List<Thing> list = map.listerThings.ThingsInGroup(ThingRequestGroup.Medicine);
                for (int i = 0; i < list.Count; i++)
                {
                    Thing thing = list[i];
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        if (Compatibility_ModMedicinePatch.CheckIfIdxAllowsMedicine(thing, idx) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            tmpMedicine.Add(thing);
                        }
                    }
                    else
                    {
                        if (medicalCare.AllowsMedicine(thing.def) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            tmpMedicine.Add(thing);
                        }
                    }
                }

                //split all available medicines in "willGrantBestChance" and "wontGrantBestChance"
                List<Thing> medicineWillGrant = new List<Thing>();
                List<Thing> medicineWontGrant = new List<Thing>();
                foreach (Thing medicine in tmpMedicine)
                {
                    float medicinesSurgerySuccessChance = curve.Evaluate(medicine.def.GetStatValueAbstract(StatDefOf.MedicalPotency, null));
                    if (medicinesSurgerySuccessChance >= necessaryValueForBestChance)
                    {
                        medicineWillGrant.Add(medicine);
                    }
                    else
                    {
                        medicineWontGrant.Add(medicine);
                    }
                }

                //sort the lists: The list with medicines that grant the 0.98f
                //if "use worst medicine that grants max success: the worst should come first
                if (lists[listIndex].operationSmartMedicineChoice)
                {
                    medicineWillGrant.SortBy((Thing x) => x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));
                }
                //else: the best should come first
                else
                {
                    medicineWillGrant.SortBy((Thing x) => -x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));
                }

                //sort the lists: The list with medicines that do not grant the 0.98f -> the best should come first
                medicineWontGrant.SortBy((Thing x) => -x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));

                //if we only want operations to happen if we reach the max surgery success chance, we only give back the medicines that will grant it
                if (lists[listIndex].operationsOnlyWhenMaxSuccessChance)
                {
                    relevantThings.AddRange(medicineWillGrant);
                }
                //Otherwise: merge the two lists, first the willGrant, then the wontGrant
                else
                {
                    relevantThings.AddRange(medicineWillGrant);
                    relevantThings.AddRange(medicineWontGrant);
                }
            }

            //If we dont want to use the worst to reach at least 0.98f: mostly a copy of vanilla
            else
            {
                List<Thing> list = map.listerThings.ThingsInGroup(ThingRequestGroup.Medicine);
                for (int i = 0; i < list.Count; i++)
                {
                    Thing thing = list[i];
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        if (Compatibility_ModMedicinePatch.CheckIfIdxAllowsMedicine(thing, idx) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            relevantThings.Add(thing);
                        }
                    }
                    else
                    {
                        if (medicalCare.AllowsMedicine(thing.def) && pawn.CanReach(thing, PathEndMode.OnCell, Danger.Deadly, false, false, TraverseMode.ByPawn) && baseValidator(thing))
                        {
                            relevantThings.Add(thing);
                        }
                    }
                }
                relevantThings.SortBy((Thing x) => -x.GetStatValue(StatDefOf.MedicalPotency, true), (Thing x) => x.Position.DistanceToSquared(billGiver.Position));
            }

            //skip original
            return false;
        }


        //a modified version of SurgeryOutcomeEffectDef.GetQuality(...): We want to get the success chance of the operation without the impact of the used medicine
        public static float GetSuccessChanceIgnoreMedicine(RecipeDef recipe, Pawn surgeon, Pawn patient/*, List<Thing> ingredients*/, BodyPartRecord part, Bill bill)
        {
            List<Thing> things = new List<Thing>();
            float result = 1f;
            if (!recipe.surgeryOutcomeEffect.comps.NullOrEmpty<SurgeryOutcomeComp>())
            {
                for (int i = 0; i < recipe.surgeryOutcomeEffect.comps.Count; i++)
                {
                    //skip if medicine related or if it clamps to 0.98f
                    if (recipe.surgeryOutcomeEffect.comps[i].GetType() == typeof(SurgeryOutcomeComp_MedicineQuality) || recipe.surgeryOutcomeEffect.comps[i].GetType() == typeof(SurgeryOutcomeComp_ClampToRange))
                    {
                        continue;
                    }
                    if (recipe.surgeryOutcomeEffect.comps[i].Affects(recipe, surgeon, patient, part))
                    {
                        recipe.surgeryOutcomeEffect.comps[i].AffectQuality(recipe, surgeon, patient, things, part, bill, ref result);
                    }
                }
            }
            return result;

        }
    }
}

