﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using Verse;
using Verse.AI;


namespace ChooseYourMedicine
{
    public class FindBestMedicine_Prefix
    {
        public static bool Prefix(Pawn healer, Pawn patient, bool onlyUseInventory, ref Thing __result)
        {
            //The best medicine that is allowed
            MedicalCareCategory medicalCare = Utility_GetList.GetTheCorrectMedicalCareCategory(patient);

            //check if we need medicine at all
            if (Medicine.GetMedicineCountToFullyHeal(patient) <= 0)
            {
                __result = null;
                return false;
            }

            //if modMedicinePatch is loaded
            int idx = 0;
            if (StartUp.modMedicinePatchLoaded)
            {
                Compatibility_ModMedicinePatch.GetIndexFromMedicalCare(medicalCare, ref idx);
                if (idx <= 1)
                {
                    __result = null;
                    return false;
                }
            }
            //if modMedicinePatch is not loaded
            else
            {
                if (medicalCare <= MedicalCareCategory.NoMeds)
                {
                    __result = null;
                    return false;
                }
            }



            IEnumerable<Thing> healerInventoryMeds = from x in healer.inventory.innerContainer.ToList()
                                                     where x.def.IsMedicine
                                                     select x;

            IEnumerable<Thing> patientInventoryMeds = from x in patient.inventory.innerContainer.ToList()
                                                      where x.def.IsMedicine
                                                      select x;

            IEnumerable<Thing> patientAndHealerInventoryMeds = healerInventoryMeds.Concat(patientInventoryMeds);
            if (onlyUseInventory)
            {
                //if modMedicinePatch is loaded
                if (StartUp.modMedicinePatchLoaded)
                {
                    for (; ; )
                    {
                        if (idx <= 1)
                        {
                            __result = null;
                            return false;
                        }
                        foreach (Thing thing in patientAndHealerInventoryMeds)
                        {
                            if (Compatibility_ModMedicinePatch.CheckIfMedicineFitsCareCategory(thing, idx) == true)
                            {
                                __result = thing;
                                return false;
                            }
                        }
                        idx--;
                    }
                }
                //if modMedicinePatch is not loaded
                else
                {
                    for (; ; )
                    {
                        if (medicalCare <= MedicalCareCategory.NoMeds)
                        {
                            __result = null;
                            return false;
                        }
                        foreach (Thing thing in patientAndHealerInventoryMeds)
                        {
                            if (GetMedicalCare.GetMedicalCareCategory(thing) == medicalCare)
                            {
                                __result = thing;
                                return false;
                            }
                        }
                        medicalCare--;
                    }
                }

            }

            else
            {
                //We dont need to search the map if we have medicine in the inventories that fits, so we just make a empty list/enumerable for now
                List<Thing> allMedsOnMap = new List<Thing>();
                IEnumerable<Thing> allMedsInAnimalsInventories = Enumerable.Empty<Thing>();

                //for loop which makes medicalCare-- at the end to find fitting medicine
                for (; ; )
                {
                    //if we could not find medicine, we return null
                    //if modMedicinePatch is loaded
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        if (idx <= 1)
                        {
                            __result = null;
                            return false;
                        }
                    }
                    //if modMedicinePatch is not loaded
                    else
                    {
                        if (medicalCare <= MedicalCareCategory.NoMeds)
                        {
                            __result = null;
                            return false;
                        }
                    }

                    //try to find fitting medicine in patient or healers inventory
                    foreach (Thing thing in patientAndHealerInventoryMeds)
                    {
                        //if modMedicinePatch is loaded
                        if (StartUp.modMedicinePatchLoaded)
                        {
                            if (Compatibility_ModMedicinePatch.CheckIfMedicineFitsCareCategory(thing, idx))
                            {
                                __result = thing;
                                return false;
                            }
                        }
                        //if modMedicinePatch is not loaded
                        else
                        {
                            if (GetMedicalCare.GetMedicalCareCategory(thing) == medicalCare)
                            {
                                __result = thing;
                                return false;
                            }
                        }
                    }

                    //if we have not checked the map for available medicines, we do it now
                    if (allMedsOnMap.NullOrEmpty())
                    {
                        allMedsOnMap = patient.Map.listerThings.ThingsInGroup(ThingRequestGroup.Medicine);
                    }

                    //try to find fitting medicine on the map
                    Thing candidate = GenClosest.ClosestThing_Global_Reachable
                        (
                         patient.Position,
                         patient.Map,
                         allMedsOnMap,
                         PathEndMode.ClosestTouch,
                         TraverseParms.For(healer, Danger.Deadly, TraverseMode.ByPawn, false, false, false),
                         searchRange(),
                         validator,
                         potencyGetter
                         );


                    //if modMedicinePatch is loaded
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        if (Compatibility_ModMedicinePatch.CheckIfMedicineFitsCareCategory(candidate, idx))
                        {
                            __result = candidate;
                            return false;
                        }

                    }
                    //if modMedicinePatch is not loaded
                    else
                    {
                        if (GetMedicalCare.GetMedicalCareCategory(candidate) == medicalCare)
                        {
                            __result = candidate;
                            return false;
                        }
                    }

                    //if we have not checked animals inventories for available medicines, we do it now
                    if (allMedsInAnimalsInventories.EnumerableNullOrEmpty())
                    {
                        foreach (Pawn pawn in healer.Map.mapPawns.SpawnedColonyAnimals)
                        {
                            if (pawn.IsForbidden(healer) || !healer.CanReach(pawn, PathEndMode.OnCell, Danger.Some, false, false, TraverseMode.ByPawn))
                            {
                                continue;
                            }
                            IEnumerable<Thing> colonyAnimalInventoryMeds = from x in pawn.inventory.innerContainer.ToList()
                                                                           where x.def.IsMedicine
                                                                           select x;
                            allMedsInAnimalsInventories = allMedsInAnimalsInventories.Concat(colonyAnimalInventoryMeds);
                        }
                    }

                    //try to find fitting medicine in animals inventories, if there is none, lower the MedicalCareCategory we are looking for and start the loop again
                    //if modMedicinePatch is loaded
                    if (StartUp.modMedicinePatchLoaded)
                    {
                        foreach (Thing candidate2 in allMedsInAnimalsInventories)
                        {
                            if (Compatibility_ModMedicinePatch.CheckIfMedicineFitsCareCategory(candidate2, idx))
                            {
                                __result = candidate2;
                                return false;
                            }

                        }
                        idx--;
                    }
                    //if modMedicinePatch is not loaded
                    else
                    {
                        foreach (Thing candidate2 in allMedsInAnimalsInventories)
                        {
                            if (GetMedicalCare.GetMedicalCareCategory(candidate2) == medicalCare)
                            {
                                __result = candidate2;
                                return false;
                            }
                        }
                        medicalCare--;
                    }
                }
            }

            //local helper methods
            float potencyGetter(Thing thing)
            {
                return thing.def.GetStatValueAbstract(StatDefOf.MedicalPotency);
            }

            bool validator(Thing m)
            {
                if (StartUp.modMedicinePatchLoaded)
                {
                    return Compatibility_ModMedicinePatch.Validator(m, healer, patient, idx);
                }
                else
                {
                    return !m.IsForbidden(healer)
                        && m.def.IsMedicine
                        && (patient.playerSettings == null || GetMedicalCare.GetMedicalCareCategory(m) == medicalCare)
                        && healer.CanReserve(m, 10, 1, null, false);
                }
            }

            float searchRange()
            {
                int range = LoadedModManager.GetMod<Settings_CYM>().settings.tendingsSearchRange;
                return range == -1 ? 9999f : (float)range;
            }

        }        
    }

    public static class GetMedicalCare
    {
        public static MedicalCareCategory GetMedicalCareCategory(Thing thing)
        {
            if (thing == null)
            {
                return MedicalCareCategory.NoMeds;
            }
            if (thing.def.IsMedicine && potencyGetter(thing) > ThingDefOf.MedicineIndustrial.GetStatValueAbstract(StatDefOf.MedicalPotency))
            {
                return MedicalCareCategory.Best;
            }
            else if (thing.def.IsMedicine && potencyGetter(thing) > ThingDefOf.MedicineHerbal.GetStatValueAbstract(StatDefOf.MedicalPotency))
            {
                return MedicalCareCategory.NormalOrWorse;
            }
            else if (thing.def.IsMedicine && potencyGetter(thing) > 0f)
            {
                return MedicalCareCategory.HerbalOrWorse;
            }
            else
            {
                return MedicalCareCategory.NoMeds;
            }

            float potencyGetter(Thing t)
            {
                return t.def.GetStatValueAbstract(StatDefOf.MedicalPotency);
            }
        }


    }

}
