﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using Verse;
using UnityEngine;

namespace ChooseYourMedicine
{
    class MedicineEntryListsCreation
    {
		public void CreateDefaultConfigs()
		{
			List<MedicineEntryList> lists = new List<MedicineEntryList>();


			//We need to create a list for each of the categories found in pawn -> health tab -> defaults: Thats 7 lists if we have ideology
			List<HediffDef> allTendableHediffs = GetAllTendableHediffs();

			//#1: Colonists
			MedicineEntryList colonists = new MedicineEntryList();
			colonists.nameOfTheList = "Kopp_CYM_ListNameColonist".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryStandard = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.Best;
				entry.careCategoryImmunityLower = MedicalCareCategory.NormalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				colonists.list.Add(entry);
			}
			colonists.operationsMaxAllowedMedCare = MedicalCareCategory.Best;
			colonists.operationSmartMedicineChoice = true;
			colonists.operationsOnlyWhenMaxSuccessChance = false;
			colonists.operationsUseBest = true;
			lists.Add(colonists);

			//#2: Imprisoned colonists
			MedicineEntryList imprisonedColonists = new MedicineEntryList();
			imprisonedColonists.nameOfTheList = "Kopp_CYM_ListNameImprisonedColonist".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryStandard = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.Best;
				entry.careCategoryImmunityLower = MedicalCareCategory.NormalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				imprisonedColonists.list.Add(entry);
			}
			imprisonedColonists.operationsMaxAllowedMedCare = MedicalCareCategory.Best;
			imprisonedColonists.operationSmartMedicineChoice = true;
			imprisonedColonists.operationsOnlyWhenMaxSuccessChance = false;
			imprisonedColonists.operationsUseBest = true;
			lists.Add(imprisonedColonists);

			//#3: Enslaved colonists
			MedicineEntryList enslavedColonists = new MedicineEntryList();
			enslavedColonists.nameOfTheList = "Kopp_CYM_ListNameEnslavedColonist".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.NoMeds;
				entry.careCategoryStandard = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryImmunityLower = MedicalCareCategory.HerbalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				enslavedColonists.list.Add(entry);
			}
			enslavedColonists.operationsMaxAllowedMedCare = MedicalCareCategory.NormalOrWorse;
			enslavedColonists.operationSmartMedicineChoice = true;
			enslavedColonists.operationsOnlyWhenMaxSuccessChance = false;
			enslavedColonists.operationsUseBest = false;
			lists.Add(enslavedColonists);

			//#4: Colony animal
			MedicineEntryList colonyAnimal = new MedicineEntryList();
			colonyAnimal.nameOfTheList = "Kopp_CYM_ListNameColonyAnimal".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.NoMeds;
				entry.careCategoryStandard = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryImmunityLower = MedicalCareCategory.HerbalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				colonyAnimal.list.Add(entry);
			}
			colonyAnimal.operationsMaxAllowedMedCare = MedicalCareCategory.NormalOrWorse;
			colonyAnimal.operationSmartMedicineChoice = true;
			colonyAnimal.operationsOnlyWhenMaxSuccessChance = false;
			colonyAnimal.operationsUseBest = false;
			lists.Add(colonyAnimal);

			//#5: Neutral animal
			MedicineEntryList neutralAnimal = new MedicineEntryList();
			neutralAnimal.nameOfTheList = "Kopp_CYM_ListNameNeutralAnimal".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.NoMeds;
				entry.careCategoryStandard = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryImmunityLower = MedicalCareCategory.HerbalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				neutralAnimal.list.Add(entry);
			}
			neutralAnimal.operationsMaxAllowedMedCare = MedicalCareCategory.HerbalOrWorse;
			neutralAnimal.operationSmartMedicineChoice = true;
			neutralAnimal.operationsOnlyWhenMaxSuccessChance = false;
			neutralAnimal.operationsUseBest = false;
			lists.Add(neutralAnimal);

			//#6: Neutral faction
			MedicineEntryList neutralFaction = new MedicineEntryList();
			neutralFaction.nameOfTheList = "Kopp_CYM_ListNameNeutralFaction".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.NoMeds;
				entry.careCategoryStandard = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryImmunityLower = MedicalCareCategory.HerbalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				neutralFaction.list.Add(entry);
			}
			neutralFaction.operationsMaxAllowedMedCare = MedicalCareCategory.HerbalOrWorse;
			neutralFaction.operationSmartMedicineChoice = true;
			neutralFaction.operationsOnlyWhenMaxSuccessChance = false;
			neutralFaction.operationsUseBest = false;
			lists.Add(neutralFaction);

			//#7: Hostile faction
			MedicineEntryList hostileFaction = new MedicineEntryList();
			hostileFaction.nameOfTheList = "Kopp_CYM_ListNameHostileFaction".Translate();
			foreach (HediffDef hediff in allTendableHediffs)
			{
				MedicineEntry entry = new MedicineEntry();
				entry.defName = hediff.defName;
				entry.label = hediff.label;
				entry.careCategoryEasy = MedicalCareCategory.NoMeds;
				entry.careCategoryStandard = MedicalCareCategory.HerbalOrWorse;
				entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
				entry.careCategoryImmunityLower = MedicalCareCategory.HerbalOrWorse;
				entry.offSetEasy = 25;
				entry.offSetProblematic = 5;
				entry.offSetImmunityLower = 20;
				entry.shouldBeHidden = false;
				entry.hasCompImmunizable = hediff.HasComp(typeof(HediffComp_Immunizable));
				hostileFaction.list.Add(entry);
			}
			hostileFaction.operationsMaxAllowedMedCare = MedicalCareCategory.HerbalOrWorse;
			hostileFaction.operationSmartMedicineChoice = true;
			hostileFaction.operationsOnlyWhenMaxSuccessChance = false;
			hostileFaction.operationsUseBest = false;
			lists.Add(hostileFaction);

			ModSettings.settings.lists = lists;
			ModSettings.Write();
		}


		public List<HediffDef> GetAllTendableHediffs()
        {
			IEnumerable<HediffDef> allTendableHediffs = new List<HediffDef>();
			allTendableHediffs = from x in DefDatabase<HediffDef>.AllDefs
								 where x.tendable
								 select x;
			allTendableHediffs = allTendableHediffs.OrderBy(c => c.label).ThenBy(c => c.defName);
			List<HediffDef> allTendableHediffs1 = new List<HediffDef>();
			allTendableHediffs1 = allTendableHediffs.ToList();

			return allTendableHediffs1;
        }

	}
}
