﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using HarmonyLib;
using RimWorld;
using Verse;
using RimWorld.Planet;
using System.Reflection;

namespace ChooseYourMedicine
{

    [StaticConstructorOnStartup]
    public class Harmony
    {
        static Harmony()
        {
            HarmonyLib.Harmony harmony = new HarmonyLib.Harmony("kopp.rimworld.chooseyourmedicine");

            //We need to alter what medicine should be used according to the settings made in the mods gui / pawns health tab
            harmony.Patch(AccessTools.Method(typeof(RimWorld.HealthAIUtility), nameof(HealthAIUtility.FindBestMedicine)),
                prefix: new HarmonyMethod(typeof(FindBestMedicine_Prefix), nameof(FindBestMedicine_Prefix.Prefix)));

            //Same as above
            harmony.Patch(AccessTools.Method(typeof(RimWorld.Medicine), nameof(Medicine.GetMedicineCountToFullyHeal)),
                prefix: new HarmonyMethod(typeof(GetMedicineCountToFullyHeal_Prefix), nameof(GetMedicineCountToFullyHeal_Prefix.Prefix)));

            //We need to change the logic in which cases the doc looks for new medicine
            harmony.Patch(AccessTools.Method(typeof(RimWorld.JobDriver_TendPatient), nameof(JobDriver_TendPatient.FindMoreMedicineToil)),
                prefix: new HarmonyMethod(typeof(FindMoreMedicineToil_Prefix), nameof(FindMoreMedicineToil_Prefix.Prefix)));

            //necessary to be able to assign a medicine manually to a hediff in the pawns health tab. And to render the icon if we did.
            harmony.Patch(GetMethod1(),
                transpiler: new HarmonyMethod(typeof(DrawHediffRow_Transpiler), nameof(DrawHediffRow_Transpiler.DrawHediffRowTranspiler)));
            //We need to change the class var lastMaxIconsTotalWidth so it does not interfere with the newly introduced button to assign medicine manually
            harmony.Patch(AccessTools.Method(typeof(RimWorld.HealthCardUtility), nameof(RimWorld.HealthCardUtility.DrawHediffListing)),
                transpiler: new HarmonyMethod(typeof(DrawHediffListing_Transpiler), nameof(DrawHediffListing_Transpiler.DrawHediffListingTranspiler)));


            //these two are necessary to remove entries from the list of hediffs that got manually assigned to a medicalCareCategory
            //this one checks every time a hediff got tended if it had the entry "one time"
            harmony.Patch(AccessTools.Method(typeof(Verse.HediffWithComps), nameof(HediffWithComps.Tended)),
                postfix: new HarmonyMethod(typeof(Tended_Postfix), nameof(Tended_Postfix.Postfix)));
            //this one will get executed every time a game is saved and removes all entries whichs hediffs dont exist anymore or whichs pawns dont exist (are alive) anymore
            harmony.Patch(AccessTools.Method(typeof(GameDataSaveLoader), nameof(GameDataSaveLoader.SaveGame)),
                prefix: new HarmonyMethod(typeof(SaveGame_Prefix), nameof(SaveGame_Prefix.Prefix)));


            //this patch will make sure that if you assigned a mod added medicine to a pawn manually and remove that mod that no errors are thrown when opening the pawns health tab
            //obviously only necessary if ModMedicinePatch is loaded
            if (ModLister.HasActiveModWithName("Mod Medicine Patch"))
            {
                harmony.Patch(AccessTools.Method(typeof(Game), nameof(Game.LoadGame)),
                    postfix: new HarmonyMethod(typeof(LoadGame_Postfix), nameof(LoadGame_Postfix.Postfix)));
            }


            //we forgot to make our changes apply to caravans as well...
            harmony.Patch(AccessTools.Method(typeof(CaravanInventoryUtility), nameof(CaravanInventoryUtility.TryGetBestMedicine)),
                prefix: new HarmonyMethod(typeof(TryGetBestMedicine_Prefix), nameof(TryGetBestMedicine_Prefix.Prefix)));


            //to change what medicine should be used for operations
            harmony.Patch(GetMethod2(),
                prefix: new HarmonyMethod(typeof(AddEveryMedicineToRelevantThings_Prefix), nameof(AddEveryMedicineToRelevantThings_Prefix.Prefix)));


            //When we assign a medicine manually to a hediff which is "better" than what the pawns settings allow, we get a 10 jobs in 10 ticks problem, the transpiler should fix this
            //This call only patches Harmony_JobDriver_TendPatient.MakeNewToil_Transpiler
            harmony.PatchAll();


            //When the doc uses no medicine to tend the first hediff, it does not use medicine for the other hediffs as well, the postfix should fix this.
            harmony.Patch(GetMethod3(),
                postfix: new HarmonyMethod(GetMethod4(), 800));

        }

        public static MethodInfo GetMethod1()
        {
            var type = AccessTools.TypeByName("RimWorld.HealthCardUtility");
            return AccessTools.Method(type, "DrawHediffRow");
        }

        public static MethodInfo GetMethod2()
        {
            var type = AccessTools.TypeByName("RimWorld.WorkGiver_DoBill");
            return AccessTools.Method(type, "AddEveryMedicineToRelevantThings");
        }

        public static MethodInfo GetMethod3()
        {
            var type = AccessTools.TypeByName("RimWorld.JobDriver_TendPatient");
            return AccessTools.Method(type, "MakeNewToils");
        }
        public static MethodInfo GetMethod4()
        {
            var type = AccessTools.TypeByName("ChooseYourMedicine.JobDriver_TendPatient_Postfix");
            return AccessTools.Method(type, "Postfix");
        }
    }
}



