﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using RimWorld;
using Verse;
using Verse.Sound;
using UnityEngine;

namespace ChooseYourMedicine
{
    public class MainTabWindow_Medicine : MainTabWindow
    {
        Vector2 scrollPosition;
        public List<MedicineEntryList> lists = new List<MedicineEntryList>();

        public override Vector2 RequestedTabSize
        {
            get
            {
                return new Vector2(1540f, 684f + 150f); //+200 for the tabs and for the copy/paste buttons - 50f
            }
        }

        //load/create the lists if not happened yet
        public override void PreOpen()
        {
            base.PreOpen();
            lists = Utility_GetList.GetList();
        }


        public override void PostClose()
        {
            base.PostClose();
            Clipboard.ClearClipboard();
            ModSettings.settings.lists = lists;
            ModSettings.Write();
        }

        public override void DoWindowContents(Rect rect)
        {
            if (listsHaveChanged)
            {
                lists = Utility_GetList.GetList();
                listsHaveChanged = false;
            }

            tabs.Clear();
            Rect rectTabs = new Rect(205f, 60f, 1070f, 65f); // to 65 from 75
            tabs = MainTabWindow_Medicine_Utility.DrawCategoryTabs(lists, rectTabs);

            for (int i = 0; i < tabs.Count(); i++)
            {
                //skipping the slaves tab if ideology is not active:
                //In MainTabWindow_Utility we dont add the "slaves" tab if ideology is not active. 
                //But to change the correct list of our config file we need to adjust the index (j).
                int j = i;
                if (j > 1 && !StartUp.ideologyActive)
                {
                    j++;
                }
                if (j > 2 && !StartUp.anomalyActive)
                {
                    j++;
                }


                if (MainTabWindow_Medicine.tab == (MainTabWindow_Medicine.Tab)i) //changed j to i
                {
                    //recache if necessary
                    if (CountHasChanged)
                    {
                        unhiddenEntriesCount = lists[j].list.Where(x => x != null && x.shouldBeHidden == false).Count();
                        hiddenEntriesCount = lists[j].list.Where(x => x != null && x.shouldBeHidden).Count();
                        allEntriesCount = unhiddenEntriesCount + hiddenEntriesCount;
                        CountHasChanged = false;
                    }


                    //scroll view stuff
                    Rect viewRect = new Rect(rect);
                    viewRect.y += 185f; //150f
                    viewRect.height -= 185f; //-150f
                    Rect scrollRect = new Rect(viewRect);
                    if (showHiddenEntries == false)
                    {
                        scrollRect.height = unhiddenEntriesCount * rowHeight + 20f + 100f; //20f so it does not look cramped. 50f for the button 
                    }
                    else
                    {
                        scrollRect.height = allEntriesCount * rowHeight + 20f + 100f; //additional 130f for the button and the gap between the two lists
                    }
                    scrollRect.width -= 20f; //we dont want a scroll thing in the width
                    Widgets.BeginScrollView(viewRect, ref scrollPosition, scrollRect, true);


                    Listing_Standard listingStandardLabel = new Listing_Standard();
                    curY = 180f;


                    //vertical lines for the unhidden entries; 2 lines in grey, the others in a custom color (light grey?)
                    float unhiddenEntriesLenght = unhiddenEntriesCount * rowHeight;
                    GUI.color = Color.grey;
                    Widgets.DrawLineVertical(211f, curY, unhiddenEntriesLenght);
                    Widgets.DrawLineVertical(1321f, curY, unhiddenEntriesLenght);
                    GUI.color = new Color(1f, 1f, 1f, 0.2f);
                    Widgets.DrawLineVertical(489f, curY, unhiddenEntriesLenght);
                    Widgets.DrawLineVertical(766f, curY, unhiddenEntriesLenght);
                    Widgets.DrawLineVertical(1043f, curY, unhiddenEntriesLenght);
                    GUI.color = Color.white;


                    //draw each unhidden entry
                    foreach (MedicineEntry entry in lists[j].list)
                    {
                        if (entry.shouldBeHidden == false)
                        {
                            DoRow(ref listingStandardLabel, unhiddenEntriesCount, entry, lists, j);
                            curY += rowHeight;
                        }
                    }


                    //button to show/hide hidden entries
                    Rect buttonShowUnhidden = new Rect(0f, curY + 50f, 210f, 34f);
                    if (Widgets.ButtonText(buttonShowUnhidden, "Kopp_CYM_ShowHidden".Translate()))
                    {
                        showHiddenEntries = !showHiddenEntries;
                        CountHasChanged = true;
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }


                    //draw the hidden section, if necessary
                    if (showHiddenEntries)
                    {
                        curY += 100f;
                        //draw vertical lines for the hidden entries; two in grey, the others in custom color (light grey?)
                        GUI.color = Color.grey;
                        float lengthHiddenEntries = hiddenEntriesCount * rowHeight;
                        Widgets.DrawLineVertical(211f, curY, lengthHiddenEntries); //this was unhiddenEntriesLength, but should probbaly be lengthHiddenEntries
                        Widgets.DrawLineVertical(1321f, curY, lengthHiddenEntries);
                        GUI.color = new Color(1f, 1f, 1f, 0.2f);
                        Widgets.DrawLineVertical(489f, curY, lengthHiddenEntries);
                        Widgets.DrawLineVertical(766f, curY, lengthHiddenEntries);
                        Widgets.DrawLineVertical(1043f, curY, lengthHiddenEntries);
                        GUI.color = Color.white;

                        //draw the hidden entries
                        foreach (MedicineEntry entry in lists[j].list)
                        {
                            if (entry.shouldBeHidden)
                            {
                                DoRow(ref listingStandardLabel, unhiddenEntriesCount, entry, lists, j);
                                curY += rowHeight;
                            }
                        }
                    }


                    Widgets.EndScrollView();


                    //copy/paste button for the columns
                    Rect buttonCopyEasy = new Rect(275f, 104f, 70f, 30f); //x: draw line vertical (211, ...) + ((draw line vertical (526, ...) - draw line vertical (211, ...) ) - (70+70+10 (button,button,gap))) / 2
                    Rect buttonPasteEasy = new Rect(355f, 104f, 70f, 30f); //x: above + 80
                    if (Widgets.ButtonText(buttonCopyEasy, "Kopp_CYM_Copy".Translate()))
                    {
                        Clipboard.CopyColumnButton(j, 0);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonPasteEasy, "Kopp_CYM_Paste".Translate()))
                    {
                        Clipboard.PasteColumnButton(j, 0);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }

                    Rect buttonCopyStandard = new Rect(553f, 104f, 70f, 30f); //x: draw line vertical (526, ...) + ((draw line vertical (691, ...) - draw line vertical (526, ...) ) - (70+70+10 (button,button,gap))) / 2
                    Rect buttonPasteStandard = new Rect(633f, 104f, 70f, 30f); //x: above + 80
                    if (Widgets.ButtonText(buttonCopyStandard, "Kopp_CYM_Copy".Translate()))
                    {
                        Clipboard.CopyColumnButton(j, 1);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonPasteStandard, "Kopp_CYM_Paste".Translate()))
                    {
                        Clipboard.PasteColumnButton(j, 1);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }

                    Rect buttonCopyProblematic = new Rect(830f, 104f, 70f, 30f); //x: ...
                    Rect buttonPasteProblematic = new Rect(910f, 104f, 70f, 30f);
                    if (Widgets.ButtonText(buttonCopyProblematic, "Kopp_CYM_Copy".Translate()))
                    {
                        Clipboard.CopyColumnButton(j, 2);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonPasteProblematic, "Kopp_CYM_Paste".Translate()))
                    {
                        Clipboard.PasteColumnButton(j, 2);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }

                    Rect buttonCopyImmunity = new Rect(1107f, 104f, 70f, 30f); //x: ...
                    Rect buttonPasteImmunity = new Rect(1187f, 104f, 70f, 30f);
                    if (Widgets.ButtonText(buttonCopyImmunity, "Kopp_CYM_Copy".Translate()))
                    {
                        Clipboard.CopyColumnButton(j, 3);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonPasteImmunity, "Kopp_CYM_Paste".Translate()))
                    {
                        Clipboard.PasteColumnButton(j, 3);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }


                    //copy/paste for the whole list and the fallBackToDefault, RefreshHediffs
                    Rect buttonCopyList = new Rect(1340f, 12f, 140f, 34f);
                    Rect buttonPasteList = new Rect(1340f, 56f, 140f, 34f);
                    Rect buttonFallBackToDefault = new Rect(0f, 12f, 140f, 34f);
                    Rect buttonCheckForNewHediffs = new Rect(0f, 56f, 140f, 34f);
                    Rect buttonOperations = new Rect(0f, 100f, 140f, 34f);

                    if (Widgets.ButtonText(buttonCopyList, "Kopp_CYM_CopyWholePage".Translate()))
                    {
                        Clipboard.CopyListButton(j);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonPasteList, "Kopp_CYM_PasteIntoThisPage".Translate()))
                    {
                        Clipboard.PasteListButton(j);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonFallBackToDefault, "Kopp_CYM_DefaultSettings".Translate()))
                    {
                        Clipboard.DefaultSettingsButton(j);
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonCheckForNewHediffs, "Kopp_CYM_UpdateList".Translate()))
                    {
                        Clipboard.RefreshHediffListsButton();
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }
                    if (Widgets.ButtonText(buttonOperations, "Kopp_CYM_Operations".Translate()))
                    {
                        Find.WindowStack.Add(new Dialog_Operations(lists));
                        SoundDefOf.Click.PlayOneShotOnCamera(null);
                    }


                    //helper button
                    Text.Font = GameFont.Tiny;
                    Rect buttonHelperOverlay = new Rect(720f, 0f, 50f, 20f);
                    if (Widgets.ButtonText(buttonHelperOverlay, "Kopp_CYM_ShowHelper".Translate()))
                    {
                        Find.WindowStack.Add(new MainTabWindow_Medicine_HelperGui());
                    }
                    Text.Font = GameFont.Small;


                    //headlines: not urgent/standard/acute/immunity <
                    //labels
                    Rect labelEasy = new Rect(212f, 146f, 278f, 40f);
                    Rect labelStandard = new Rect(490f, 146f, 277f, 40f);
                    Rect labelProblematic = new Rect(767f, 146f, 277f, 40f);
                    Rect labelImmunityLower = new Rect(1044f, 146f, 277f, 40f);
                    Text.Anchor = TextAnchor.MiddleCenter;
                    Widgets.Label(labelEasy, "Kopp_CYM_RowEasyLabel".Translate());
                    Widgets.Label(labelStandard, "Kopp_CYM_RowStandardLabel".Translate());
                    Widgets.Label(labelProblematic, "Kopp_CYM_RowProblematicLabel".Translate());
                    Widgets.Label(labelImmunityLower, "Kopp_CYM_RowImmunityLabel".Translate());

                    //lines
                    GUI.color = new Color(1f, 1f, 1f, 0.2f);
                    Widgets.DrawLineHorizontal(211f, 145f, 1110f);
                    GUI.color = Color.grey;
                    Widgets.DrawLineHorizontal(0f, 185f, 1480f); //changed the length to be the whole page. Prior: x 211f; length 1110f;
                    GUI.color = Color.white;

                    GUI.color = new Color(1f, 1f, 1f, 0.2f);
                    Widgets.DrawLineVertical(211f, 145f, 40f);
                    Widgets.DrawLineVertical(489f, 145f, 40f);
                    Widgets.DrawLineVertical(766f, 145f, 40f);
                    Widgets.DrawLineVertical(1043f, 145f, 40f);
                    Widgets.DrawLineVertical(1321f, 145f, 40f);
                    GUI.color = Color.white;
                    Text.Font = GameFont.Small;
                    Text.Anchor = TextAnchor.UpperLeft;

                    //tooltips
                    TooltipHandler.TipRegion(labelEasy, "Kopp_CYM_RowEasyTooltip".Translate());
                    if (Mouse.IsOver(labelEasy))
                    {
                        Widgets.DrawHighlight(labelEasy);
                    }
                    TooltipHandler.TipRegion(labelStandard, "Kopp_CYM_RowStandardTooltip".Translate());
                    if (Mouse.IsOver(labelStandard))
                    {
                        Widgets.DrawHighlight(labelStandard);
                    }
                    TooltipHandler.TipRegion(labelProblematic, "Kopp_CYM_RowProblematicTooltip".Translate());
                    if (Mouse.IsOver(labelProblematic))
                    {
                        Widgets.DrawHighlight(labelProblematic);
                    }
                    TooltipHandler.TipRegion(labelImmunityLower, "Kopp_CYM_RowImmunityTooltip".Translate());
                    if (Mouse.IsOver(labelImmunityLower))
                    {
                        Widgets.DrawHighlight(labelImmunityLower);
                    }

                }
            }

        }



        //curY should be 150f for the beginning of the first listing (unhidden entries)
        public static void DoRow(ref Listing_Standard listingStandardLabel, int count, MedicineEntry entry, List<MedicineEntryList> lists, int j)
        {
            float curYSlider = curY + 12f;
            float curYButtons = curY + 9f;
            float curYDropDown = curY + 8f;

            Rect rectLabel = new Rect(0f, curY, 210f, count * rowHeight + 20f);


            float curX = 212f + 48f; //48f = (276f (one block) minus size for icon, gap and slider )* 0.5

            ////easy
            Rect rectEasyDropdown = new Rect(curX, curYDropDown, 30f, 30f);
            curX += 30f + 20f;
            Rect rectEasySlider = new Rect(curX, curYSlider, 130f, 30f);
            Rect rectEasyTooltip = new Rect(212f, curY +5f, 278f, rowHeight);

            curX = 490f + 123f;

            ////standard
            Rect rectStandardDropdown = new Rect(curX, curYDropDown, 30f, 30f);
            Rect rectStandardTooltip = new Rect(490f, curY + 5f, 277f, rowHeight);

            curX = 767f + 48f;

            ////problematic
            Rect rectProblematicDropdown = new Rect(curX, curYDropDown, 30f, 30f);
            curX += 30f + 20f;
            Rect rectProblematicSlider = new Rect(curX, curYSlider, 130f, 30f);
            Rect rectProblematicTooltip = new Rect(767f, curY + 5f, 277f, rowHeight);

            curX = 1044f + 48f;

            ////immunityLower
            Rect rectImmunityDropdown = new Rect(curX, curYDropDown, 30f, 30f);
            curX += 30f + 20f;
            Rect rectImmunitySlider = new Rect(curX, curYSlider, 130f, 30f);
            Rect rectImmunityLowerTooltip = new Rect(1044f, curY+5f, 277f, rowHeight);

            //copy/paste buttons
            Rect buttonCopy1 = new Rect(1330f, curYButtons, 70f, 30f);
            Rect buttonPaste = new Rect(1410f, curYButtons, 70f, 30f);

            //for the horizontal lines
            float horizontalLine = curY + 5f;

            //for the info button
            Rect rectInfoButton = new Rect(190f, curY + 14f, 20f, 20f);

            //for different background
            Rect rectHighlightRow = new Rect(rectLabel);
            rectHighlightRow.height = rowHeight;
            rectHighlightRow.y += 5f;
            rectHighlightRow.width = 1480f;


            //hediff labels
            listingStandardLabel.Begin(rectLabel);
            listingStandardLabel.Gap(14f);
            listingStandardLabel.Label(entry.label.CapitalizeFirst(), -1, "defName: " + entry.defName);
            listingStandardLabel.Gap(6f);
            listingStandardLabel.End();

            //dropdowns and sliders
            //if modMedicinePatch is loaded
            if (StartUp.modMedicinePatchLoaded)
            {
                Compatibility_ModMedicinePatch.MainTabWindow1(rectStandardDropdown, ref entry, rectEasyDropdown, rectEasySlider, rectProblematicDropdown, rectProblematicSlider, rectImmunityDropdown, rectImmunitySlider);
            }
            else
            {
                MedicalCareDropdownMenu.MedicalCareSelectButtonStandard(rectStandardDropdown, entry);
                if (entry.hasCompImmunizable)
                {
                    MedicalCareDropdownMenu.MedicalCareSelectButtonEasy(rectEasyDropdown, entry);
                    entry.offSetEasy = (int)Widgets.HorizontalSlider(rectEasySlider, entry.offSetEasy, 0f, 100f, false, (entry.offSetEasy.ToString() + " %").Colorize(yellowCustom), "0", "100");

                    MedicalCareDropdownMenu.MedicalCareSelectButtonProblematic(rectProblematicDropdown, entry);
                    entry.offSetProblematic = (int)Widgets.HorizontalSlider(rectProblematicSlider, entry.offSetProblematic, 0f, 100f, false, (entry.offSetProblematic.ToString() + " %").Colorize(yellowCustom), "0", "100");

                    MedicalCareDropdownMenu.MedicalCareSelectButtonImmunityLower(rectImmunityDropdown, entry);
                    entry.offSetImmunityLower = (int)Widgets.HorizontalSlider(rectImmunitySlider, entry.offSetImmunityLower, 0f, 100f, false, (entry.offSetImmunityLower.ToString() + " %").Colorize(yellowCustom), "0", "100");
                }
            }

            //tooltips
            if (entry.hasCompImmunizable)
            {
                TooltipHandler.TipRegion(rectEasyTooltip, "Kopp_CYM_RowEasyTooltip".Translate());
                if (Mouse.IsOver(rectEasyTooltip))
                {
                    Widgets.DrawHighlight(rectEasyTooltip);
                }
                TooltipHandler.TipRegion(rectProblematicTooltip, "Kopp_CYM_RowProblematicTooltip".Translate());
                if (Mouse.IsOver(rectProblematicTooltip))
                {
                    Widgets.DrawHighlight(rectProblematicTooltip);
                }
                TooltipHandler.TipRegion(rectImmunityLowerTooltip, "Kopp_CYM_RowImmunityTooltip".Translate());
                if (Mouse.IsOver(rectImmunityLowerTooltip))
                {
                    Widgets.DrawHighlight(rectImmunityLowerTooltip);
                }
            }
            TooltipHandler.TipRegion(rectStandardTooltip, "Kopp_CYM_RowStandardTooltip".Translate());
            if (Mouse.IsOver(rectStandardTooltip))
            {
                Widgets.DrawHighlight(rectStandardTooltip);
            }

            //copy/paste buttons
            int i = lists[j].list.IndexOf(entry);
            if (Widgets.ButtonText(buttonCopy1, "Kopp_CYM_Copy".Translate()))
            {
                Clipboard.CopyRowButton(j, i);
                SoundDefOf.Click.PlayOneShotOnCamera(null);
            }
            if (Widgets.ButtonText(buttonPaste, "Kopp_CYM_Paste".Translate()))
            {
                Clipboard.PasteRowButton(j, i);
                SoundDefOf.Click.PlayOneShotOnCamera(null);
            }

            //invisible "hide/unhide" button (when clicking the label of a hediff)
            Rect testRect = new Rect(rectLabel);
            testRect.height = rowHeight;
            testRect.width -= 20f; //-20f; we need space for the info button
            testRect.y += 5f;
            if (Mouse.IsOver(testRect))
            {
                Widgets.DrawHighlight(testRect);
            }
            int test = i;
            int test2 = j;
            if (Widgets.ButtonInvisible(testRect))
            {
                List<FloatMenuOption> list = new List<FloatMenuOption>();
                if (entry.shouldBeHidden == false)
                {
                    list.Add(new FloatMenuOption("Kopp_CYM_Hide".Translate(), delegate ()
                    {
                        lists[test2].list[test].shouldBeHidden = true;
                        CountHasChanged = true;

                    }, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));
                }
                else
                {
                    list.Add(new FloatMenuOption("Kopp_CYM_Unveil".Translate(), delegate ()
                    {
                        lists[test2].list[test].shouldBeHidden = false;
                        CountHasChanged = true;

                    }, MenuOptionPriority.Default, null, null, 0f, null, null, true, 0));
                }
                Find.WindowStack.Add(new FloatMenu(list));
            }

            //horizontal lines
            GUI.color = new Color(1f, 1f, 1f, 0.2f);
            Widgets.DrawLineHorizontal(0f, horizontalLine, 1480f);
            GUI.color = Color.white;

            //info button
            //Will this check the whole dataBase for the def 60times per second?
            Widgets.InfoCardButton(rectInfoButton, DefDatabase<HediffDef>.GetNamedSilentFail(entry.defName));

            //highlight row if mouse is over
            if (Mouse.IsOver(rectHighlightRow))
            {
                Widgets.DrawHighlight(rectHighlightRow);
            }
        }


        public static Tab tab;
        public enum Tab
        {
            tab0,
            tab1,
            tab2,
            tab3,
            tab4,
            tab5,
            tab6,
        }

        public List<TabRecord> tabs = new List<TabRecord>();
        public static bool listsHaveChanged { get; set; } = false;
        public static bool showHiddenEntries = false;
        public static float curY;
        public static int unhiddenEntriesCount = 0;
        public static int hiddenEntriesCount = 0;
        public static int allEntriesCount = 0;
        public static float rowHeight = 36f;      
        public static Color yellowCustom = new Color(1f, 0.92156863f, 0.015686275f, 0.5f); //copied from unityEngine, adapted alpha value
        public static bool CountHasChanged { get; set; } = true;

    }
}
