﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Verse;
using RimWorld;
using UnityEngine;

namespace ChooseYourMedicine
{
    [StaticConstructorOnStartup]
    public static class StartUp
    {
        static StartUp()
        {
            List<MedicineEntryList> lists = Utility_GetList.GetList();
            //if modMedicinePatch is not loaded
            if (!ModLister.HasActiveModWithName("Mod Medicine Patch"))
            {
                //textures
                LongEventHandler.ExecuteWhenFinished(delegate
            {
                careTextures = new Texture2D[5];
                careTextures[0] = ContentFinder<Texture2D>.Get("UI/Icons/Medical/NoCare", true);
                careTextures[1] = ContentFinder<Texture2D>.Get("UI/Icons/Medical/NoMeds", true);
                careTextures[2] = ThingDefOf.MedicineHerbal.uiIcon;
                careTextures[3] = ThingDefOf.MedicineIndustrial.uiIcon;
                careTextures[4] = ThingDefOf.MedicineUltratech.uiIcon;
            });

                randomTextures = new Texture2D[3];
                randomTextures[0] = ContentFinder<Texture2D>.Get("CYM/clear", true);
                randomTextures[1] = ContentFinder<Texture2D>.Get("CYM/once", true);
                randomTextures[2] = ContentFinder<Texture2D>.Get("CYM/forever", true);

                stringMedicalCare = new string[5] { "Kopp_CYM_Clear".Translate(), ((MedicalCareCategory)1).GetLabel().CapitalizeFirst(), ((MedicalCareCategory)2).GetLabel().CapitalizeFirst(), ((MedicalCareCategory)3).GetLabel().CapitalizeFirst(), ((MedicalCareCategory)4).GetLabel().CapitalizeFirst() };

                //If ModMedicinePatch gets removed from the loadOrder and have some non-vanilla meds in the list, we cant open the gui... So we need to loop through and remove them
                //List<MedicineEntryList> lists = Utility_GetList.GetList();
                foreach (MedicineEntryList list in lists)
                {
                    foreach (MedicineEntry entry in list.list)
                    {
                        if (entry.careCategoryEasy > MedicalCareCategory.Best)
                        {
                            entry.careCategoryEasy = MedicalCareCategory.NormalOrWorse;
                        }
                        if (entry.careCategoryStandard > MedicalCareCategory.Best)
                        {
                            entry.careCategoryStandard = MedicalCareCategory.NormalOrWorse;
                        }
                        if (entry.careCategoryProblematic > MedicalCareCategory.Best)
                        {
                            entry.careCategoryProblematic = MedicalCareCategory.NormalOrWorse;
                        }
                        if (entry.careCategoryImmunityLower > MedicalCareCategory.Best)
                        {
                            entry.careCategoryImmunityLower = MedicalCareCategory.NormalOrWorse;
                        }
                    }
                }
                modMedicinePatchLoaded = false;

            }
            //ModMedicinePatch is loaded
            else
            {
                modMedicinePatchLoaded = true;
                TraverseStuff.GetNecessaryStuff();
                Compatibility_ModMedicinePatch.CreateOrderedCareList();
                Compatibility_ModMedicinePatch.TryToFixGuiEntries();

                randomTextures = new Texture2D[3];
                randomTextures[0] = ContentFinder<Texture2D>.Get("CYM/clear", true);
                randomTextures[1] = ContentFinder<Texture2D>.Get("CYM/once", true);
                randomTextures[2] = ContentFinder<Texture2D>.Get("CYM/forever", true);
            }

            //Need to check the mod settings if the mods main button should be visible or not
            if (CYMDefOf.CYM_Medicine?.buttonVisible != LoadedModManager.GetMod<Settings_CYM>().settings?.showMainTabButton)
            {
                CYMDefOf.CYM_Medicine.buttonVisible = LoadedModManager.GetMod<Settings_CYM>().settings.showMainTabButton;
            }

            //Need to check if the amount of medicine a colonist is allowed to carry equals what is stated in the mod settings
            if (InventoryStockGroupDefOf.Medicine.max != LoadedModManager.GetMod<Settings_CYM>().settings?.maxMedicineCarryCount)
            {
                InventoryStockGroupDefOf.Medicine.max = LoadedModManager.GetMod<Settings_CYM>().settings.maxMedicineCarryCount;
            }

            //Update the tabs list, if necessary
            UpdateFrom1_0_15(lists);
        }


        //v1.0.16 is changing the number of lists/tabs (colonists, colonyAnimals, entities, ...) we have 
        //Thats not a issue if ppl use the mod for the first time, but when they used it before
        //So we have to check that and adapt: Make a copy of the old list/config (<11 entries); create default config (11 entries) and save it; replace certain entries from newly created config with ones from oldConfig
        //Well... Maybe we should have just created 4 new entries and added them to the current list/config...
        public static void UpdateFrom1_0_15(List<MedicineEntryList> lists)
        {
            if (lists.Count() == 7)
            {
                Clipboard.RefreshHediffListsButton();
                List<MedicineEntryList> oldList = new List<MedicineEntryList>();
                for (int i = 0; i < lists.Count(); i++)
                {
                    ; MedicineEntryList list = new MedicineEntryList(lists[i]);
                    oldList.Add(list);
                }
                MedicineEntryListsCreation defaultListsCreation = new MedicineEntryListsCreation();
                defaultListsCreation.CreateDefaultConfigs();
                ModSettings.Write();

                lists = Utility_GetList.GetList();
                lists.Replace(lists[0], oldList[0]);
                lists.Replace(lists[1], oldList[1]);
                lists.Replace(lists[2], oldList[2]);
                lists.Replace(lists[4], oldList[3]);
                lists.Replace(lists[6], oldList[5]);
                lists.Replace(lists[7], oldList[6]);
                lists.Replace(lists[9], oldList[4]);

                ModSettings.Write();
            }
        }



        public static Texture2D[] randomTextures;
        public static Texture2D[] careTextures;
        public static string[] stringMedicalCare;
        //a copy of vanillas Widgets.AltTexture, but not so bright
        public static readonly Texture2D AltTexture_Custom = SolidColorMaterials.NewSolidColorTexture(new Color(0.41f, 0.41f, 0.41f, 0.05f));
        public static bool modMedicinePatchLoaded = false;
        //public static readonly FloatRange SliderRange = new FloatRange(0f, 100f); //will need when changing the Widgets.HorizontalSlider to the not-obsolete version. But then we probably should make range (0f, 1f)...
        public static bool ideologyActive = ModsConfig.IdeologyActive;
        public static bool anomalyActive = ModsConfig.AnomalyActive;
        public static GameFont tabFontSize = (ModsConfig.IdeologyActive && ModsConfig.AnomalyActive) ? GameFont.Tiny : GameFont.Small;
        public static List<string> tabNames = new List<string>
        {
            "Kopp_CYM_ListNameColonist".Translate(),
            "Kopp_CYM_ListNameImprisonedColonist".Translate(),
            "Kopp_CYM_ListNameEnslavedColonist".Translate(),
            "Kopp_CYM_ListNameGhouls".Translate(),
            "Kopp_CYM_ListNameColonyAnimal".Translate(),

            "Kopp_CYM_ListNameFriendlyFaction".Translate(),
            "Kopp_CYM_ListNameNeutralFaction".Translate(),
            "Kopp_CYM_ListNameHostileFaction".Translate(),

            "Kopp_CYM_ListNameOtherHumans".Translate(),
            "Kopp_CYM_ListNameNeutralAnimal".Translate(),
            "Kopp_CYM_ListNameEntities".Translate()
        };

    }

}

